-- ------------------------------------------------------------------------------
-- RP Tags 
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Utils-Config: Utilities for reading and registering configuration values

local RPTAGS     = RPTAGS;

table.insert(RPTAGS.startup, function(state)
  local Const      = RPTAGS.const;
  local Utils      = RPTAGS.utils;
  
  if not Utils.config then Utils.config = {} end;
  local Config = Utils.config;
  
  local Trp3Keys        = Const.CONFIG.KEYS;
  local Default     = Const.CONFIG.DEFAULTS;
  
  for key, _ in pairs(Default)
  do  local db = RP_TagsDB.settings;
      if not db[key] then db[key] = {} end;
      db[key].key = Trp3Keys[key];
      db[key].value = db[key].value or Default[key];
      db[key].default = Default[key];
      db[key].init = nil;
  end;
         
        -- return the value of a config setting
  local function getConfig(key) if not key then return nil; end;
    if   RP_TagsDB.settings[key]
    then return RP_TagsDB.settings[key].value or RP_TagsDB.settings[key].default
    else return nil
    end;
  end;  
  
  local function setConfig(key, value) if not key then return nil; end;
    if   RP_TagsDB.settings[key]
    then RP_TagsDB.settings[key].value = value;
         return value;
    else return nil;
    end; -- if;
  end; -- function
  
  local function resetConfig(key) if not key then return nil; end; 
    if    RP_TagsDB.settings[key]
    then RP_TagsDB.settings[key].value = RP_TagsDB.settings[key].default;
         return RP_TagsDB.settings[key].default;
    else return nil;
    end; --if
  end; -- function
  
  local function getDefault(key) if not key then return nil; end;
    if RP_TagsDB.settings[key]
    then return RP_TagsDB.settings[key].default or Default[key];
    else return nil;
    end;
  end; -- function
  
  local function resetConfigGroup(group) for _, s in ipairs(RPTAGS.const.CONFIG.GROUPS[group]) do resetConfig(s) end; end; -- function
        
  local function tagEditBox(key) if not key then return nil; end;
    if RP_TagsDB.settings[key]
    then StaticPopup_Show("RPTAGS_TAG_EDIT_BOX", nil, nil, { setting = key, current = RP_TagsDB.settings[key].value, });
         return true;
    else return false;
    end; -- if
  end; -- function

  if state.trp3
  then
    local Register = { config = { key = TRP3_API.configuration.registerConfigKey, }, };

    if   not RP_TagsDB.TRP3_Config_Imported
    then for setting, key in pairs(Trp3Keys)
         do Register.config.key(key, Default[setting]); -- trp3 has to be aware of these tags to get the values from them
            local value = TRP3_API.configuration.getValue(key);
            if   RP_TagsDB.settings[setting].key == key -- just make sure we are importing the same setting
            then RP_TagsDB.settings[setting].value = value;
                 -- TRP3_API.configuration.setValue(nil); -- clean it out of the user's settings -- hey that sounds dangerous!
            end; --if
            end; -- for setting, key
            RP_TagsDB.TRP3_Config_Imported = true;
            RPTAGS.cache.TRP3_Config_Imported = true;
    end -- if not already imported
  end;

  RPTAGS.const.CONFIG.KEYS = nil; -- don't need these at this point

  -- Functions available under RPTAGS.utils.config
  --
  Config.tagedit    = tagEditBox;
  Config.get        = getConfig;
  Config.set        = setConfig;
  Config.reset      = resetConfig;
  Config.default    = getDefault;
  Config.resetgroup = resetConfigGroup;

  -- keybindings
  BINDING_HEADER_RPTAGS              = RPTAGS.utils.locale.loc("APP_NAME");
  BINDING_NAME_RPTAGS_DISABLE_RPUF   = RPTAGS.utils.locale.loc("KEYBIND_DISABLE_RPUF");
  BINDING_NAME_RPTAGS_HIDE_IN_COMBAT = RPTAGS.utils.locale.loc("KEYBIND_HIDE_IN_COMBAT");
  BINDING_NAME_RPTAGS_LOCK_FRAMES    = RPTAGS.utils.locale.loc("KEYBIND_LOCK_FRAMES");
  BINDING_NAME_RPTAGS_IC_OOC         = RPTAGS.utils.locale.loc("KEYBIND_IC_OOC");
  BINDING_NAME_RPTAGS_TAG_EDITOR     = RPTAGS.utils.locale.loc("KEYBIND_TAG_EDITOR");

  return "config"

end); -- startup function
