-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Help: the /rptags command

local RPTAGS     = RPTAGS;

table.insert(RPTAGS.startup, function(state) -- startup function

  if not RPTAGS.utils.help then RPTAGS.utils.help = {} end;

  local Const      = RPTAGS.const;
  local Utils      = RPTAGS.utils;
  local Cache      = RPTAGS.cache;
  
  local loc        = Utils.locale.loc;
  local hilite     = Utils.text.hilite;
  local Config     = Utils.config;
  local notify     = Utils.text.notify;

  local setReference = Utils.help.reference
  
  RPTAGS.cache.helpOptions = {};

  -- initialize help system -------------------------------------------------------------------------------------------------------------------
  --
  -- by this time we have all of the tags loaded, localized
  -- and all of the help topics in RPTAGS.cache.help

  -- topics first
  for _, helpText in ipairs(RPTAGS.cache.help)
  do  local topic = helpText:match("<h1>(.-)</h1>");
      if   topic 
      then table.insert(RPTAGS.cache.helpOptions, { text = "|cffdddd00" .. topic .. "|r", value = topic, } );
           RPTAGS.help[topic] = "<html><body>" .. hilite(helpText, true) .. "</body></html>";
      else print("Ora, I can't find a topic in", text:sub(1, 50));
      end  -- if topic
  end;

  -- tags next
  for _, tagCategory in ipairs(RPTAGS.cache.TAG_DATA)
  do table.insert(RPTAGS.cache.helpOptions, { text = tagCategory.title, value = tagCategory.key, });
     local tagHelp = { "<h1>"..tagCategory.title.."</h1>",
                       "<br /><p>" .. tagCategory.help .. "</p><br />"}
     for _, tag in ipairs(tagCategory.tags)
     do  if     tag.name and tag.desc 
         then   table.insert(tagHelp, "<h3>["..tag.name.."]</h3><p> &nbsp; &nbsp; "..tag.desc .. "</p>")
         elseif tag.title
         then   table.insert(tagHelp, "<br /><h2>" .. tag.title .. "</h2><br />")
         end    -- if tag.name
     RPTAGS.help[tagCategory.key] = "<html><body>" .. hilite(table.concat(tagHelp), true) .. "</body></html>";
     end; -- for tag
  end; -- for tagCategory
       
  -- for _, h in ipairs(RPTAGS.cache.helpOption) do print("Ora, topic", h.text, "is", RPTAGS.help[h.text]:sub(1,50)) end;

  -- slash commands ---------------------------------------------------------------------------------------------------------------------------
  SLASH_RPTAGS1          = loc("APP_SLASH");
  SLASH_RPTAGS2          = loc("APP_SLASH_SHORT");
  SLASH_RPTAGS3          = loc("APP_SLASH_SHORTER");

  SlashCmdList["RPTAGS"] = 
    function(a) 
      if     a == ""           or
             a == "help"       then InterfaceOptionsFrame_Show(); InterfaceOptionsFrame_OpenToCategory(RPTAGS_Main_OptionsPanel)
      elseif a == "config"     or
             a == "options"    or
             a == "settings"   then InterfaceOptionsFrame_Show(); InterfaceOptionsFrame_OpenToCategory(RPTAGS_General_OptionsPanel);
      elseif a == "uf"         or
             a == "rpuf"       or 
             a == "unitframes" then InterfaceOptionsFrame_Show(); InterfaceOptionsFrame_OpenToCategory(RPTAGS_RPUnitFrames_OptionsPanel);
      elseif a == "colors"     or
             a == "color"      then InterfaceOptionsFrame_Show(); InterfaceOptionsFrame_OpenToCategory(RPTAGS_Colors_OptionsPanel);
      elseif a == "layout"     then InterfaceOptionsFrame_Show(); InterfaceOptionsFrame_OpenToCategory(RPTAGS_Layout_OptionsPanel);
      elseif a == "tags"       or
             a == "panels"     then InterfaceOptionsFrame_Show(); InterfaceOptionsFrame_OpenToCategory(RPTAGS_Tags_OptionsPanel);
      elseif a == "keybind"    or
             a == "key"        or
             a == "bind"       then KeyBindingFrame_LoadUI(); KeyBindingFrame.mode = 1; ShowUIPanel(KeyBindingFrame);
      elseif a == "version"    then notify(RPTAGS.utils.text.version());
      end;
  end;
   
  return "help";
end); -- startup function  
