-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

-- Init: program initalization

local addOnName, RPTAGS = ...;
RPTAGS.addOnName = addOnName;

for _, item in ipairs({ "cache", "const", "get", "help", "locale", "startup", "utils", }) do RPTAGS[item] = {} end;

local startupFrame = CreateFrame("frame")
startupFrame:RegisterEvent("ADDON_LOADED");
startupFrame:SetScript("OnEvent", 
  function(self, event, addOnLoaded)
    if addOnLoaded == addOnName
       then -- make sure our saved variables exist
            if not RP_TagsDB          then RP_TagsDB          = {} end;
            if not RP_TagsDB.settings then RP_TagsDB.settings = {} end;
 
            local state      = {};
            local rpClients  = {};
            local unitFrames = {};

            if _G["ElvUI"]    then state.elvui = true table.insert(unitFrames, "elvui") end;
            if true           then state.rpuf  = true table.insert(unitFrames, "rpuf")  end; -- "if true" because by definition RPUF is always available
            if _G["TRP3_API"] then state.trp3  = true table.insert(rpClients,  "trp3" ) end;
            if _G["mrp"]      then state.mrp   = true table.insert(rpClients,  "mrp"  ) end;
         -- if _G["xrp"]      then state.xrp   = true table.insert(rpClients,  "xrp"  ) end;

            -- logic time!

             -- rpClient logic
            if     #rpClients >  1 then state.err = "TOO_MANY_MSP"  for _, addon in ipairs(rpClients) do state[addon] = false; end
            elseif #rpClients == 0 then state.err = "NO_MSP_CLIENT"                                                            end;
            
            state.rpClients       = rpClients;
            state.unitFrames      = unitFrames;
            RPTAGS.cache.startLog = { "This table contains a list of each startup routine excuted, in the order in which they were executed." };
            RPTAGS.cache.state    = state;

            for _, startupRoutine in ipairs(RPTAGS.startup)
            do  local started = startupRoutine(state)
                table.insert(RPTAGS.cache.startLog, started)
                if started:match("%-break$") and state.err then break end; -- we have enough to give error messages
            end; -- for

            StaticPopupDialogs["RPTAGS_NO_MSP_CLIENT_FOUND"] = 
              { text      = RPTAGS.utils.locale.loc("NO_MSP_CLIENT"),
                button1   = RPTAGS.utils.locale.loc("ERROR_ACKNOWLEDGED"),
                showAlert = true, whileDead = true, timeout = 0, };

            StaticPopupDialogs["RPTAGS_TOO_MANY_MSP_CLIENTS"] = 
              { text      = RPTAGS.utils.locale.loc("TOO_MANY_MSP") .. table.concat(rpClients), 
                button1   = RPTAGS.utils.locale.loc("ERROR_ACKNOWLEDGED"),
                showAlert = true, whileDead = true, timeout = 0, };

            if state.err then StaticPopup_Show("RPTAGS_" .. state.err); end;

            -- if the user wants the login message, show it
            if RPTAGS.utils.config.get("LOGIN_MESSAGE")   then RPTAGS.utils.text.notify(RPTAGS.utils.text.version(state)); end; -- if
            if RPTAGS.utils.config.get("CHANGES_MESSAGE") and RPTAGS.utils.config.get("LOGIN_MESSAGE") 
            then RPTAGS.utils.text.notify(RPTAGS.utils.text.changes(state)); end; -- if

       end; -- if addOnLoaded
    end); -- function
_G.RPTAGS = RPTAGS;
