-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

-- KeyBinds: keybinds

local RPTAGS     = RPTAGS;

table.insert(RPTAGS.startup, function(state) -- startup function

  local notify = RPTAGS.utils.text.notify;
  local Config = RPTAGS.utils.config;
  local loc    = RPTAGS.utils.locale.loc;

  local function keybind_disableRPUF()
    Config.set("DISABLE_RPUF", not(Config.get("DISABLE_RPUF")))
    RPTAGS.utils.rpuf.allFrames.disable();
  end;

  local function keybind_hideInCombat()
    if Config.get("DISABLE_RPUF")
    then notify("ERROR_KEYBIND_NO_RPUF")
    else Config.set("RPUF_HIDE", not(Config.get("RPUF_HIDE")))
         RPTAGS.utils.rpuf.allFrames.visibility();
    end;
  end;

  local function keybind_lockFrames()
    if Config.get("DISABLE_RPUF")
    then notify("ERROR_KEYBIND_NO_RPUF")
    else Config.set("LOCK_FRAMES", not(Config.get("LOCK_FRAMES")))
         RPTAGS.utils.rpuf.allFrames.lock();
    end;
  end;

  local function keybind_IC_OOC()
    if state.trp3 and TRP3_API
    then 
      TRP3_API.dashboard.switchStatus();
        local you_are_now = loc("NOTIFY_KEYBIND_IC_OOC") .. "|cff";

        if TRP3_API.dashboard.isPlayerIC()
           then you_are_now = you_are_now .. RPTAGS.utils.config.get("COLOR_IC") .. loc("IS_IC_VERBOSE") .. "|r."
           else you_are_now = you_are_now .. RPTAGS.utils.config.get("COLOR_OOC") .. loc("IS_OOC_VERBOSE") .. "|r."
        end;

      notify(you_are_now)
    elseif state.mrp and mrp and msp
    then
      if msp.my.FC == "1" -- ooc
         then mrp.Commands.ic()
         else mrp.Commands.ooc()
      end;
    else notify("ERROR_KEYBIND_IC_OOC");
    end;
  end;

  local function keybind_tagEditor()
    if     Config.get("DISABLE_RPUF")
    then   notify("ERROR_KEYBIND_NO_RPUF")
    elseif RPTAGS_TagEdit:IsVisible()
    then   notify("NOTIFY_KEYBIND_TAG_EDITOR_OPEN")
    elseif RP_TagsDB.editor and RP_TagsDB.editor.last 
    then   RPTAGS.utils.rpuf.tags.edit(RP_TagsDB.editor.last)
           notify("NOTIFY_KEYBIND_TAG_EDITOR")
    else   notify("ERROR_KEYBIND_TAG_EDITOR")
           InterfaceOptionsFrame_Show(); InterfaceOptionsFrame_OpenToCategory(RPTAGS_Tags_OptionsPanel)
    end;    
  end;
  
  if not RPTAGS.utils.keybinds then RPTAGS.utils.keybinds = {} end;
  RPTAGS.utils.keybinds.hideInCombat = keybind_hideInCombat;
  RPTAGS.utils.keybinds.ic_ooc       = keybind_IC_OOC;
  RPTAGS.utils.keybinds.tagEditor    = keybind_tagEditor;
  RPTAGS.utils.keybinds.lockFrames   = keybind_lockFrames;
  RPTAGS.utils.keybinds.disableRPUF  = keybind_disableRPUF;

  return "keybinds";
end); -- startup function  
