local RPTAGS = RPTAGS;

table.insert(RPTAGS.startup, function(state)
  local loc = RPTAGS.utils.locale.loc;

  -- these set up the pulldown menus for the config page
  local    weights,     heights, sizebuffs, profilesizes, _;
        _, weights      = RPTAGS.utils.format.kg(73);
        _, heights      = RPTAGS.utils.format.cm(168);
        _, sizebuffs    = RPTAGS.utils.format.sizebuff(8212);
        _, profilesizes = RPTAGS.utils.format.sizewords(7, BIG_PROFILE);
        _, unsup        = RPTAGS.utils.format.unsup("rp:tag");
  
  -- used below
  local TRUNC_OR_WRAP = {
  { loc("OPT_LINE_TRUNCATE"),     "TRUNCATE",     },
  { loc("OPT_LINE_WORD_TRUNC"),   "WORD_TRUNC",   },
  { loc("OPT_LINE_WRAP_ONCE"),    "WRAP_ONCE",    },
  { loc("OPT_LINE_WRAP_TWICE"),   "WRAP_TWICE",   },
  { loc("OPT_LINE_WRAP_FOREVER"), "WRAP_FOREVER", },
  };
  
  local RPUF_LAYOUTS = {
  { loc("RPUF_FULL"),       "FULL",      },
  { loc("RPUF_ABRIDGED"),   "ABRIDGED",  },
  { loc("RPUF_COMPACT"),    "COMPACT",   },
  { loc("RPUF_PAPERDOLL"),  "PAPERDOLL", },
  { loc("RPUF_THUMBNAIL"),  "THUMBNAIL", },
--  { loc("RPUF_HIDDEN"),     "HIDDEN",    },
  };

  local RPUF_LAYOUTS_SMALL = {
  { loc("RPUF_ABRIDGED"),   "ABRIDGED",  },
  { loc("RPUF_COMPACT"),    "COMPACT",   },
  { loc("RPUF_THUMBNAIL"),  "THUMBNAIL", },
--  { loc("RPUF_HIDDEN"),     "HIDDEN",    },
  };
  
  RPTAGS.cache.layoutOptions = {
    RPUF_Player       = { choices = RPUF_LAYOUTS,       config = "PLAYERLAYOUT" },
    RPUF_Target       = { choices = RPUF_LAYOUTS,       config = "TARGETLAYOUT" },
    RPUF_Focus        = { choices = RPUF_LAYOUTS_SMALL, config = "FOCUSLAYOUT"  },
    RPUF_TargetTarget = { choices = RPUF_LAYOUTS_SMALL, config = "FOCUSLAYOUT"  },
  };

  local ALIGN =  { 
    HORIZONTAL = { { loc("LEFT"),        "LEFT",        }, 
                   { loc("CENTER"),      "CENTER",      }, 
                   { loc("RIGHT"),       "RIGHT",       }, },
    VERTICAL   = { { loc("TOP"),         "TOP",         },
                   { loc("CENTER"),      "CENTER",      },
                   { loc("BOTTOM"),      "BOTTOM",      }, },
    CORNERS    = { { loc("TOPLEFT"),     "TOPLEFT",     },
                   { loc("TOPRIGHT"),    "TOPRIGHT",    },
                   { loc("CENTER"),      "CENTER",      },
                   { loc("BOTTOMLEFT"),  "BOTTOMLEFT",  },
                   { loc("BOTTOMRIGHT"), "BOTTOMRIGHT", }, },
    FULL       = { { loc("TOPLEFT"),     "TOPLEFT",     },
                   { loc("TOP"),         "TOP",         },
                   { loc("TOPRIGHT"),    "TOPRIGHT",    },
                   { loc("LEFT"),        "LEFT",        },
                   { loc("CENTER"),      "CENTER",      },
                   { loc("RIGHT"),       "RIGHT",       },
                   { loc("BOTTOMLEFT"),  "BOTTOMLEFT",  },
                   { loc("BOTTOM"),      "BOTTOM",      }, 
                   { loc("BOTTOMRIGHT"), "BOTTOMRIGHT", }, },
  };

  -- this works with the above to finish setting up the pulldowns
  local VALUES = { -- yeah there are probably easier ways to do this but whatever
    UNITS_HEIGHT = { 
    { heights["CM"],              "CM" },
    { heights["CM_FT_IN"],        "CM_FT_IN" },
    { heights["CM_IN"],           "CM_IN" },
    { heights["FT_IN"],           "FT_IN" },
    { heights["FT_IN_CM"],        "FT_IN_CM" },
    { heights["IN"],              "IN" }, 
    { heights["IN_CM"],           "IN_CM" },
    { heights["HEIGHT_PASSTHRU"], "HEIGHT_PASSTHRU" },
   },
    UNITS_WEIGHT = { 
    { weights["KG"],              "KG" },
    { weights["KG_LB"],           "KG_LB" },
    { weights["KG_ST_LB"],        "KG_ST_LB" },
    { weights["LB_KG"],           "LB_KG",}, 
    { weights["LB_ST"],           "LB_ST",},
    { weights["ST_LB"],           "ST_LB",},
    { weights["ST_LB_KG"],        "ST_LB_KG",},
    { weights["ST_LB_LB_KG"],     "ST_LB_LB_KG" },
    { weights["WEIGHT_PASSTHRU"], "WEIGHT_PASSTHRU" },
   },
    PROFILESIZE_FMT = { 
    { profilesizes["WRD_BRC_NUM"], "WRD_BRC_NUM" },
    { profilesizes["WRD_PAR_NUM"], "WRD_PAR_NUM" },
    { profilesizes["BRC_WRD_NUM"], "BRC_WRD_NUM" },
    { profilesizes["PAR_WRD"],     "PAR_WRD" },
    { profilesizes["WRD"],         "WRD" },
    { profilesizes["BRC_LTR_NUM"], "BRC_LTR_NUM" }, 
    { profilesizes["BRC_LTR"],     "BRC_LTR" }, 
    { profilesizes["LTR"],         "LTR",},
    { profilesizes["BRC_NUM"],     "BRC_NUM",},
    { profilesizes["NUM"],         "NUM",},
   },
    SIZEBUFF_FMT = { 
    { sizebuffs["PCT"],          "PCT" },
    { sizebuffs["PCT_BRC"],      "PCT_BRC" },
    { sizebuffs["PCT_BUFF"],     "PCT_BUFF" },
    { sizebuffs["BUFF_PCT"],     "BUFF_PCT" },
    { sizebuffs["BUFF"],         "BUFF" },
    { sizebuffs["BUFF_BRC"],     "BUFF_BRC" },
    { sizebuffs["BUFF_PCT_BRC"], "BUFF_PCT_BRC" },
    },
    GLANCE_COLON = { 
    { loc("CONFIG_GLANCE_COLON_COLON"),   ": "  , },
    { loc("CONFIG_GLANCE_COLON_COMMAS"), ", "  , },
    { loc("CONFIG_GLANCE_COLON_DASHES"),  " - " , },
    { loc("CONFIG_GLANCE_COLON_SLASHES"), " / " , },
    { loc("CONFIG_GLANCE_COLON_SPACES"),  " "   , }, 
    },
    GLANCE_DELIM = { 
    { loc("CONFIG_GLANCE_DELIM_COMMAS"),     ", ",  },
    { loc("CONFIG_GLANCE_DELIM_NEWLINES"),   "\n",  },
    { loc("CONFIG_GLANCE_DELIM_SEMICOLONS"), "; ",  },
    { loc("CONFIG_GLANCE_DELIM_SLASHES"),    " / ", },
    { loc("CONFIG_GLANCE_DELIM_SPACES"),     ", ",  },
    },
    REF_TOPIC        = { }, -- filled in later
    NAME_WRAP        = TRUNC_OR_WRAP,
    TITLE_WRAP       = TRUNC_OR_WRAP,
    GLANCE_WRAP      = TRUNC_OR_WRAP,
    GLANCETITLE_WRAP = TRUNC_OR_WRAP,
    MISC_WRAP        = TRUNC_OR_WRAP,
    CURR_WRAP        = TRUNC_OR_WRAP,
    PLAYERLAYOUT     = RPUF_LAYOUTS,
    TARGETLAYOUT     = RPUF_LAYOUTS,
    FOCUSLAYOUT      = RPUF_LAYOUTS_SMALL,
    TARGETTARGETLAYOUT = RPUF_LAYOUTS_SMALL,
    PARTYLAYOUT      = RPUF_LAYOUTS,
    RAIDLAYOUT       = RPUF_LAYOUTS_SMALL,
    PARTY_ORIENTATION = { { loc("HORIZONTAL"), "HORIZONTAL", }, { loc("VERTICAL"), "VERTICAL", }, },
    RAID_GRID = {
      { loc("5X8"), "5X8", }, { loc("4X10"), "4X10", }, { loc("2X20"), "2X20", },
      { loc("8X5"), "8X5", }, { loc("10X4"), "10X4", }, { loc("20X2"), "20X2", },
    },
    UNSUP_TAG = {
    { "?", "?" },
    { "??", "??" },
    { "???", "???" },
    { "[?]", "[?]", },
    { "|cffdddd00?|r", "|cffdddd00?|r", },
    { "|cffdd0000?|r", "|cffdd0000?|r", },
    { "|cffdddd00[?]|r", "|cffdddd00[?]|r", },
    { "|cffdd0000[?]|r", "|cffdd0000[?]|r", },
    { "|TInterface\\RAIDFRAME\\ReadyCheck-NotReady:0|t", "|TInterface\\RAIDFRAME\\ReadyCheck-NotReady:0|t", },
    { loc("CONFIG_UNSUP_NONE"),  "",                },
    },
    STATUS_ALIGN = ALIGN.CORNERS,
    STATUS_TEXTURE = {
    { loc("BLANK"),           "BLANK", },
    { loc("SHADED"),          "SHADED", },
    { loc("SOLID"),           "SOLID", },
    { loc("BAR"), "BAR", },
    { loc("RAID"), "RAID", },
    { loc("SKILLS"), "SKILLS", },
    }, -- status_texture
    PORTRAIT_STYLE = {
    { loc("PORTRAIT_STYLE_LAYOUT"), "LAYOUT", },
    { loc("PORTRAIT_STYLE_FACE"),   "FACE",   },
    { loc("PORTRAIT_STYLE_BODY"),   "BODY",   },
    },
    RPUF_WITH_ELVUI  = {
    { loc("MODE_PARALLEL"), true,       },
    { loc("MODE_ELVUI"),    false,      },
    },
    RPUF_BACKDROP    = {
    { loc("BACKDROP_ORIGINAL"), "ORIGINAL", },
    { loc("BACKDROP_BLIZZTOOLTIP"), "BLIZZTOOLTIP", },
    },
  }; -- closes the VALUES table
  
  RPTAGS.const.CONFIG.VALUES = VALUES;

  local function valuesToMenu(menuName) if not menuName then return nil end; if not VALUES[menuName] then return nil end;
    local menu = {};
    for _, item in ipairs(VALUES[menuName]) do local text, value = unpack(item); table.insert(menu, { text = text, value = value }); end; 
    return menu;
  end; -- function

  RPTAGS.utils.config.valuesToMenu = valuesToMenu;

  return "core-menus";
end); -- startup function 
