-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Utils-Config: Utilities for reading and registering configuration values

local RPTAGS     = RPTAGS;


table.insert(RPTAGS.startup, function(state) -- startup function
  local Utils      = RPTAGS.utils;
    
  local valuesToMenu = RPTAGS.utils.config.valuesToMenu;
    
  if not RPTAGS.cache then RPTAGS.cache = {} end;
  if not Utils.config then Utils.config = {} end;
  if not Utils.help   then Utils.help   = {} end;
  local Config = Utils.config;
  
  local loc         = Utils.locale.loc;
  local hi          = Utils.text.hilite;
  local refreshAll  = Utils.tags.refreshAll;
  local notify      = Utils.text.notify;

  -- rpuf utilities
  
  local resizeFrame   = Utils.rpuf.frame.resize;
  local resizeAll     = Utils.rpuf.allFrames.resize;
  local setBackdrop   = Utils.rpuf.allFrames.backdrop;
  local setHide     = Utils.rpuf.allFrames.visibility;
  local lockFrames  = Utils.rpuf.allFrames.lock;
  local scaleFrames = Utils.rpuf.allFrames.scale;
  local setBgs      = Utils.rpuf.allFrames.setbg;
  local disableFrames = Utils.rpuf.allFrames.disable;

  local version     = Utils.text.v(); -- yes, meant as a function call

  local r, g, b, lastElement, anchor, xOffset, yOffset, relAnchor;

  -- libraries
  --
  local PC_Dropdown = LibStub("PhanxConfig-Dropdown");
  local PC_Color    = LibStub("PhanxConfig-ColorPicker");
  local PC_Check    = LibStub("PhanxConfig-Checkbox");
  local PC_Text     = LibStub("PhanxConfig-EditBox");
  local PC_Slider   = LibStub("PhanxConfig-Slider");

  local LSM         = LibStub("LibSharedMedia-3.0")
  local monoFont    = CreateFont("RPTAGS_MonoFont");
        monoFont:SetFont(LSM:Fetch("font", "SourceCodePro Regular"), 12);
  local helpFont     = CreateFont("RPTAGS_HelpFont");
        helpFont:CopyFontObject(GameFontNormal);
  local helpFont_font, helpFont_baseSize, _ = helpFont:GetFont();
  local helpFont_H1 = CreateFont("RPTAGS_HelpFont_H1");
        helpFont_H1:SetFont(helpFont_font, helpFont_baseSize + 6);
        helpFont_H1:SetTextColor(0.9, 0.9, 0);
  local helpFont_H2 = CreateFont("RPTAGS_HelpFont_H2");
        helpFont_H2:SetFont(helpFont_font, helpFont_baseSize + 3);
        helpFont_H2:SetTextColor(0, 0.9, 0.9)
  local helpFont_P = CreateFont("RPTAGS_HelpFont_P");
        helpFont_P:SetFont(helpFont_font, helpFont_baseSize);
        helpFont_P:SetSpacing(3);
        helpFont_P:SetTextColor(0.9, 0.9, 0.9)

  -- handlers
  local function openEditor(self) return RPTAGS.utils.rpuf.tags.edit(self.setting) end;

  -- we're going to create all the panels first
  --
  local OptionsPanel    = CreateFrame("Frame", "RPTAGS_Main_OptionsPanel"                      ); 
  local GeneralPanel    = CreateFrame("Frame", "RPTAGS_General_OptionsPanel",      OptionsPanel); 
  local UnitFramesPanel = CreateFrame("Frame", "RPTAGS_RPUnitFrames_OptionsPanel", OptionsPanel);
  local ColorsPanel     = CreateFrame("Frame", "RPTAGS_Colors_OptionsPanel",       OptionsPanel);
  local LayoutPanel     = CreateFrame("Frame", "RPTAGS_Layout_OptionsPanel",       OptionsPanel);
  local TagsPanel       = CreateFrame("Frame", "RPTAGS_Tags_OptionsPanel",         OptionsPanel);

        OptionsPanel.name = loc("CONFIG_MENU_REFERENCE");
        OptionsPanel.widgets = {};
        RPTAGS.cache.configPanel = OptionsPanel;
        InterfaceOptions_AddCategory(OptionsPanel);
      
  local OptionsTitle = OptionsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
        OptionsTitle:SetPoint("TOPLEFT", 16, -16);
        OptionsTitle:SetJustifyH("LEFT");
        OptionsTitle:SetText(loc("CONFIG_PAGE_REFERENCE"));
        OptionsPanel.title = OptionsTitle;
        OptionsPanel.version = OptionsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        OptionsPanel.version:SetPoint("TOPRIGHT", -16, -16);
        OptionsPanel.version:SetJustifyH("RIGHT");
        OptionsPanel.version:SetText(version);

  local OptionsInstructions = OptionsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        OptionsInstructions:SetPoint("TOPLEFT", OptionsTitle, "BOTTOMLEFT", 0, -16)
        OptionsInstructions:SetWidth(375);
        OptionsInstructions:SetText(hi(loc("CONFIG_DESC_REFERENCE")));
        OptionsInstructions:SetWordWrap(true);
        OptionsInstructions:SetJustifyH("LEFT");
        OptionsInstructions:SetJustifyV("TOP");

  local ReferenceFrame = CreateFrame("Frame", "RPTAGS_ReferenceFrame", OptionsPanel);
        ReferenceFrame:SetPoint("TOPLEFT", OptionsInstructions, "BOTTOMLEFT", 0, -16);
        ReferenceFrame:SetPoint("BOTTOMRIGHT", OptionsPanel, "BOTTOMRIGHT", -16, 16);
        ReferenceFrame:SetBackdrop(RPTAGS.const.BACKDROP.BLIZZTOOLTIP);
        ReferenceFrame:SetBackdropColor(0.1, 0.1, 0.1, 0.9);

  local ReferenceScrollFrame = CreateFrame( "ScrollFrame", "RPTAGS_ReferenceScrollFrame", ReferenceFrame, "UIPanelScrollFrameTemplate" )
        ReferenceScrollFrame:SetPoint("TOPLEFT", ReferenceFrame, "TOPLEFT", 16, -16);
        ReferenceScrollFrame:SetPoint("BOTTOMRIGHT", ReferenceFrame, "BOTTOMRIGHT", -16, 32);
        ReferenceScrollFrame:EnableMouse(true);

  local ReferencePreviewBar = ReferenceFrame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        ReferencePreviewBar:SetTextColor(0.7, 0.7, 0.7);
        ReferencePreviewBar:SetPoint("BOTTOMLEFT", ReferenceFrame, "BOTTOMLEFT", 16, 16)
        ReferencePreviewBar:SetPoint("BOTTOMRIGHT", ReferenceFrame, "BOTTOMRIGHT", -16, 16)
        ReferencePreviewBar:SetJustifyH("LEFT");
        ReferencePreviewBar:SetJustifyV("BOTTOM");

  local ReferenceSimpleHtml = CreateFrame("SimpleHTML", "RPTAGS_ReferenceSimpleHTML", ReferenceScrollFrame);
        ReferenceSimpleHtml:SetSize(475, 20);
        ReferenceSimpleHtml:SetFontObject(RPTAGS_HelpFont_Small);
        ReferenceSimpleHtml:SetFontObject("p", RPTAGS_HelpFont_P);
        ReferenceSimpleHtml:SetFontObject("h1", RPTAGS_HelpFont_H1);
        ReferenceSimpleHtml:SetFontObject("h2", RPTAGS_HelpFont_H2);
        ReferenceSimpleHtml:SetFontObject("h3", RPTAGS_MonoFont);
        ReferenceSimpleHtml:SetText("help loading");

  local ReferenceTopic = PC_Dropdown:New(OptionsPanel, "", loc("CONFIG_REFERENCE_TOPIC_TT"), RPTAGS.cache.helpOptions);
        ReferenceTopic.setting = "REFERENCE_TOPIC"
        ReferenceTopic:SetPoint("BOTTOMRIGHT", ReferenceFrame, "TOPRIGHT", 0, 8);
        ReferenceTopic:SetWidth(185);
        ReferenceTopic.OnValueChanged = 
          function(self, value) 
            Config.set(self.setting, value); 
            ReferenceSimpleHtml:SetText(RPTAGS.help[value])
            end; -- function
        OptionsPanel.widgets.ReferenceTopic = ReferenceTopic;

  local function openTagHelpTopic(tag)
        if RPTAGS.utils.tags.exists(tag)
        then local tagHelp = RPTAGS.cache.tagcache[tag].help;
                   ReferenceTopic:SetValue(tagHelp);
                   Config.set("REFERENCE_TOPIC", tagHelp);
                   ReferenceSimpleHtml:SetText(RPTAGS.help[tagHelp]);
        end;
  end;

  local function openHelpTopic(topic)
        if RPTAGS.help[topic]
        then ReferenceTopic:SetValue(topic);
             Config.set("REFERENCE_TOPIC", topic);
             ReferenceSimpleHtml:SetText(RPTAGS.help[topic])
        end;
  end;

  local function openConfigMenu(menu)
        if     menu == "reference" then InterfaceOptionsFrame_OpenToCategory(RPTAGS_Main_OptionsPanel)
        elseif menu == "general"   then InterfaceOptionsFrame_OpenToCategory(RPTAGS_General_OptionsPanel);
        elseif menu == "rpuf"      then InterfaceOptionsFrame_OpenToCategory(RPTAGS_RPUnitFrames_OptionsPanel);
        elseif menu == "colors"    then InterfaceOptionsFrame_OpenToCategory(RPTAGS_Colors_OptionsPanel);
        elseif menu == "layout"    then InterfaceOptionsFrame_OpenToCategory(RPTAGS_Layout_OptionsPanel);
        elseif menu == "tags"      then InterfaceOptionsFrame_OpenToCategory(RPTAGS_Tags_OptionsPanel);
        elseif menu == "keybind"   then KeyBindingFrame_LoadUI(); KeyBindingFrame.mode = 1; ShowUIPanel(KeyBindingFrame);

        end;
  end;

  StaticPopupDialogs["RPTAGS_OPEN_URL"] = {
    OnShow     = 
      function(self, data)
        self.text:SetFormattedText(hi("RPTAGS Link Window\n\nCopy the following URL for " .. data.name .. " and paste it into your browser, then close this window."));
        self.editBox:SetText(data.url);
        self.editBox:SetAutoFocus(true);
        self.editBox:HighlightText();
        self.editBox:SetWidth(300);
        self.button1:SetPoint("RIGHT", self, "RIGHT", -12, 0);
        self.text:SetJustifyH("LEFT");
        self.text:SetSpacing(3);
      end,
    text = "",
    wide = true,
    closeButton = true,
    button1     = "Got it", 
    exclusive  = true,
    timeout    = 60,
    whileDead  = true,
    hasEditBox = true,
    OnAccept = function(self) self:Hide() end,
    enterClicksFirstButton = true,
    EditBoxOnEnterPressed = function(self) self:GetParent():Hide() end,
    EditBoxOnEscapePressed = function(self) self:GetParent():Hide() end,
    hideOnEscape = true,
    };

  local function openUrl(site, url, name)
        if site and loc("URL_" .. site:upper()) ~= "" then url = loc("URL_" .. site:upper()) end;
        StaticPopup_Show("RPTAGS_OPEN_URL", nil, nil, { url = url, name = name, });
  end;

        ReferenceSimpleHtml:SetScript("OnHyperlinkClick",
          function(f, link, text, button, ...)
            local  protocol =  link:match("^(%a+):");
            local  dest     =  link:match(":(.+)");
            if     protocol == "rptag"                       then    openTagHelpTopic(dest);
            elseif protocol == "rphelp"                      then      openHelpTopic(topic);
            elseif protocol == "rpconfig"                    then      openConfigMenu(dest);
            elseif protocol == "http" or protocol == "https" then openUrl(dest, link, text);
            end;
          end) -- function
        ReferenceSimpleHtml:SetHyperlinksEnabled(true);
      
        ReferenceSimpleHtml:SetScript("OnHyperlinkEnter",
          function(f, link, text, button, ...)
            local  protocol = link:match("^(%a+):");
            local  dest     = link:match(":(.+)");

            if     protocol == "http"     or 
                   protocol == "https"    then   SetCursor("TAXI_CURSOR");     ReferencePreviewBar:SetText("Click to get the link for the "..text.." web site.");
            elseif protocol == "rptag"    then   SetCursor("INSPECT_CURSOR");  ReferencePreviewBar:SetText("Click to view help for the "..text.." tag."        );
            elseif protocol == "rphelp"   then   SetCursor("TRAINER_CURSOR");  ReferencePreviewBar:SetText("Click to view help about "..text.."."              );
            elseif protocol == "rpconfig" then   SetCursor("INTERACT_CURSOR"); ReferencePreviewBar:SetText("Click to go to the "..text.." screen."             );
            end;

          end);

        ReferenceSimpleHtml:SetScript("OnHyperlinkLeave",
          function(f, link, text, button, ...)
            SetCursor(nil);
            ReferencePreviewBar:SetText("");
          end);

        ReferenceScrollFrame:SetScrollChild( ReferenceSimpleHtml );


  OptionsPanel.refresh = 
    function(self)
      -- update the current values
      for _, widget in pairs(self.widgets) do widget:SetValue(Config.get(widget.setting)) end;
      -- update the currently displayed topic
      ReferenceSimpleHtml:SetText(RPTAGS.help[Config.get("REFERENCE_TOPIC")]);
    end;

  OptionsPanel.default =
    function(self)
      for _, widget in pairs(self.widgets) do RPTAGS.utils.config.reset(widget.setting) end;
      notify("OPTIONS_ARE_RESET");
    end; -- ##

  -- general options panel --------------------------------------------------------------------------------------------------------------------
        GeneralPanel.name = loc("CONFIG_MENU_GENERAL");
        GeneralPanel.widgets = {};
        GeneralPanel.parent = OptionsPanel.name;
        InterfaceOptions_AddCategory(GeneralPanel);
  local GeneralTitle = GeneralPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
        GeneralTitle:SetPoint("TOPLEFT", 16, -16);
        GeneralTitle:SetJustifyH("LEFT");
        GeneralTitle:SetText(loc("CONFIG_PAGE_GENERAL"));
        GeneralPanel.title = GeneralTitle;
        GeneralPanel.version = GeneralPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        GeneralPanel.version:SetPoint("TOPRIGHT", -16, -16);
        GeneralPanel.version:SetJustifyH("RIGHT");
        GeneralPanel.version:SetText(version);
  local GeneralInstructions = GeneralPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        GeneralInstructions:SetPoint("TOPLEFT", GeneralTitle, "BOTTOMLEFT", 0, -16)
        GeneralInstructions:SetWidth(575);
        GeneralInstructions:SetText(hi(loc("CONFIG_DESC_GENERAL")));
        GeneralInstructions:SetWordWrap(true);
        GeneralInstructions:SetJustifyH("LEFT");
        GeneralInstructions:SetJustifyV("TOP");

    -- display
  local DisplayHeading = GeneralPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
        DisplayHeading:SetPoint("TOPLEFT", GeneralInstructions, "BOTTOMLEFT", 0, -16);
        DisplayHeading:SetJustifyH("LEFT"); -- ##
        DisplayHeading:SetText(loc("CONFIG_H1_DISPLAY"))
  local ShowLoginMessage = PC_Check:New(GeneralPanel, loc("CONFIG_LOGIN_MESSAGE"), hi(loc("CONFIG_LOGIN_MESSAGE_TT")));
        ShowLoginMessage:SetPoint("TOPLEFT", DisplayHeading, "BOTTOMLEFT", 0, -8);
        ShowLoginMessage.setting = "LOGIN_MESSAGE";
        ShowLoginMessage.Text:SetWidth(160);
        GeneralPanel.widgets.ShowLoginMessage = ShowLoginMessage;
        ShowLoginMessage.OnValueChanged = function(self, value) Config.set(self.setting, value); GeneralPanel.widgets.ShowChangesMessage:SetEnabled(value) end;
  local ShowChangesMessage = PC_Check:New(GeneralPanel, loc("CONFIG_CHANGES_MESSAGE"), hi(loc("CONFIG_CHANGES_MESSAGE_TT")));
        ShowChangesMessage:SetPoint("LEFT", ShowLoginMessage.Text, "RIGHT", 5,  0);
        ShowChangesMessage.setting = "CHANGES_MESSAGE";
        ShowChangesMessage.Text:SetWidth(160);
        ShowChangesMessage.OnValueChanged = function(self, value) Config.set(self.setting, value); end;
        GeneralPanel.widgets.ShowChangesMessage = ShowChangesMessage;
  local ShowLineBreaks = PC_Check:New(GeneralPanel, loc("CONFIG_LINEBREAKS"), hi(loc("CONFIG_LINEBREAKS_TT")));
        ShowLineBreaks:SetPoint("LEFT", ShowChangesMessage.Text, "RIGHT", 5,  0);
        ShowLineBreaks.setting = "LINEBREAKS";
        ShowLineBreaks.Text:SetWidth(160);
        ShowLineBreaks.OnValueChanged = function(self, value) Config.set(self.setting, value); refreshAll(); end;
        GeneralPanel.widgets.ShowLineBreaks = ShowLineBreaks;
     
        -- parsing
  local ParseHeading = GeneralPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
        ParseHeading:SetPoint("TOPLEFT", ShowLoginMessage, "BOTTOMLEFT", 0, -16);
        ParseHeading:SetJustifyH("LEFT");
        ParseHeading:SetText(loc("CONFIG_H1_PARSE"))
  local ParseHeightWeight = PC_Check:New(GeneralPanel, loc("CONFIG_PARSE_HW"), hi(loc("CONFIG_PARSE_HW_TT")));
        ParseHeightWeight:SetPoint("TOPLEFT", ParseHeading, "BOTTOMLEFT", 0, -8);
        ParseHeightWeight.setting = "PARSE_HW";
        ParseHeightWeight.Text:SetWidth(160);
        ParseHeightWeight.OnValueChanged = function(self, value) Config.set(self.setting, value); refreshAll(); end;
        GeneralPanel.widgets.ParseHeightWeight = ParseHeightWeight;
  local ParseGender = PC_Check:New(GeneralPanel, loc("CONFIG_PARSE_GENDER"), hi(loc("CONFIG_PARSE_GENDER_TT")));
        ParseGender:SetPoint("LEFT", ParseHeightWeight.Text, "RIGHT", 5,  0);
        ParseGender.setting = "PARSE_GENDER";
        ParseGender.Text:SetWidth(160);
        GeneralPanel.widgets.ParseGender = ParseGender;
        ParseGender.OnValueChanged = function(self, value) Config.set(self.setting, value); 
        GeneralPanel.widgets.ParseAdultGenders:SetEnabled(value);
        ColorsPanel.widgets.COLOR_THEY:SetEnabled(value);
        refreshAll(); end;
  local ParseAge = PC_Check:New(GeneralPanel, loc("CONFIG_PARSE_AGE"), hi(loc("CONFIG_PARSE_AGE_TT")));
        ParseAge:SetPoint("TOPLEFT", ParseHeightWeight, "BOTTOMLEFT", 0, 0);
        ParseAge.setting = "PARSE_AGE";
        ParseAge.Text:SetWidth(160);
        ParseAge.OnValueChanged = function(self, value) Config.set(self.setting, value); refreshAll(); end;
        GeneralPanel.widgets.ParseAge = ParseAge;
  local ParseAdultGenders = PC_Check:New(GeneralPanel, loc("CONFIG_ADULT_GENDERS"), hi(loc("CONFIG_ADULT_GENDERS_TT")));
        ParseAdultGenders:SetPoint("LEFT", ParseAge.Text, "RIGHT", 5, 0);
        ParseAdultGenders.setting = "ADULT_GENDERS";
        ParseAdultGenders.Text:SetWidth(160);
        ParseAdultGenders.OnValueChanged = function(self, value) Config.set(self.setting, value); refreshAll(); end;
        GeneralPanel.widgets.ParseAdultGenders = ParseAdultGenders;

  anchor, lastElement, relAnchor, xOffset, yOffset = "TOPLEFT", ParseAge, "BOTTOMLEFT", 0, -8;
  local NotePatterns = {};
  for i = 1, 3
      do local note = PC_Text:New(GeneralPanel, loc("CONFIG_NOTE_" ..i.."_STRING"), hi(loc("CONFIG_NOTE_"..i.."_STRING_TT")));
         note:SetPoint(anchor, lastElement, relAnchor, xOffset, yOffset);
         note.setting = "NOTE_"..i.."_STRING";
         note:SetWidth(185);
         note.OnTextChanged = function(self, text) Config.set(self.setting, text); refreshAll(); end;
         NotePatterns[i] = note; anchor, lastElement, relAnchor, xOffset, yOffset = "LEFT", note, "RIGHT", 15, 0;
         GeneralPanel.widgets["NotePattern" .. i] = note;
      end;

  -- formats
  local FormatsHeading = GeneralPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
        FormatsHeading:SetPoint("TOPLEFT", NotePatterns[1], "BOTTOMLEFT", 0, -16);
        FormatsHeading:SetJustifyH("LEFT");
        FormatsHeading:SetText(loc("CONFIG_H1_FORMATS"))
  local HeightFormat = PC_Dropdown:New(GeneralPanel, loc("CONFIG_UNITS_HEIGHT"), loc("CONFIG_UNITS_HEIGHT_TT"), valuesToMenu("UNITS_HEIGHT"));
        HeightFormat.setting = "UNITS_HEIGHT";
        HeightFormat:SetPoint("TOPLEFT", FormatsHeading, "BOTTOMLEFT", 0, -16);
        HeightFormat:SetWidth(185);
        HeightFormat.OnValueChanged = function(self, value) Config.set(self.setting, value); refreshAll() end;
        GeneralPanel.widgets.HeightFormat = HeightFormat;
  local WeightFormat = PC_Dropdown:New(GeneralPanel, loc("CONFIG_UNITS_WEIGHT"), loc("CONFIG_UNITS_WEIGHT_TT"), valuesToMenu("UNITS_WEIGHT"));
        WeightFormat:SetPoint("LEFT", HeightFormat, "RIGHT", 15, 0);
        WeightFormat:SetWidth(185);
        WeightFormat.setting = "UNITS_WEIGHT";
        WeightFormat.OnValueChanged = function(self, value) Config.set(self.setting, value); refreshAll() end;
        GeneralPanel.widgets.WeightFormat = WeightFormat;
  local SizeBuffFormat = PC_Dropdown:New(GeneralPanel, loc("CONFIG_SIZEBUFF_FMT"), loc("CONFIG_SIZEBUFF_FMT_TT"), valuesToMenu("SIZEBUFF_FMT"));
        SizeBuffFormat:SetPoint("LEFT", WeightFormat, "RIGHT", 15, 0);
        SizeBuffFormat:SetWidth(185);
        SizeBuffFormat.setting = "SIZEBUFF_FMT";
        SizeBuffFormat.OnValueChanged = function(self, value) Config.set(self.setting, value); refreshAll() end;
        GeneralPanel.widgets.SizeBuffFormat = SizeBuffFormat;
  local GlanceDelimiterFormat = PC_Dropdown:New(GeneralPanel, loc("CONFIG_GLANCE_DELIM"), loc("CONFIG_GLANCE_DELIM_TT"), valuesToMenu("GLANCE_DELIM"));
        GlanceDelimiterFormat.setting = "GLANCE_DELIM";
        GlanceDelimiterFormat:SetPoint("TOPLEFT", HeightFormat, "BOTTOMLEFT", 0, -16);
        GlanceDelimiterFormat:SetWidth(185);
        GlanceDelimiterFormat.OnValueChanged = function(self, value) Config.set(self.setting, value); refreshAll() end;
        GeneralPanel.widgets.GlanceDelimiterFormat = GlanceDelimiterFormat;
  local GlanceColonFormat = PC_Dropdown:New(GeneralPanel, loc("CONFIG_GLANCE_COLON"), loc("CONFIG_GLANCE_COLON_TT"), valuesToMenu("GLANCE_COLON"));
        GlanceColonFormat.setting = "GLANCE_COLON";
        GlanceColonFormat:SetPoint("LEFT", GlanceDelimiterFormat, "RIGHT", 15, 0);
        GlanceColonFormat:SetWidth(185);
        GlanceColonFormat.OnValueChanged = function(self, value) Config.set(self.setting, value); refreshAll() end;
        GeneralPanel.widgets.GlanceColonFormat = GlanceColonFormat;
  local ProfileSizeFormat = PC_Dropdown:New(GeneralPanel, loc("CONFIG_PROFILESIZE_FMT"), loc("CONFIG_PROFILESIZE_FMT_TT"), valuesToMenu("PROFILESIZE_FMT"));
        ProfileSizeFormat:SetPoint("LEFT", GlanceColonFormat, "RIGHT", 15, 0);
        ProfileSizeFormat:SetWidth(185);
        ProfileSizeFormat.setting = "PROFILESIZE_FMT";
        ProfileSizeFormat.OnValueChanged = function(self, value) Config.set(self.setting, value); refreshAll() end;
        GeneralPanel.widgets.ProfileSizeFormat = ProfileSizeFormat;
  local UnsupTagFormat = PC_Dropdown:New(GeneralPanel, loc("CONFIG_UNSUP_TAG"), loc("CONFIG_UNSUP_TAG_TT"), valuesToMenu("UNSUP_TAG"));
        UnsupTagFormat.setting = "UNSUP_TAG";
        UnsupTagFormat:SetPoint("TOPLEFT", GlanceDelimiterFormat, "BOTTOMLEFT", 0, 0)
        UnsupTagFormat:SetWidth(185);
        UnsupTagFormat.OnValueChanged = function(self, value) Config.set(self.setting, value); refreshAll() end;
        GeneralPanel.widgets.UnsupTagFormat = UnsupTagFormat;

  GeneralPanel.refresh = 
    function(self)
      -- update the current values
      for _, widget in pairs(self.widgets) do widget:SetValue(Config.get(widget.setting)) end;
      -- make sure the text boxes are usable
      for i = 1, 3 do NotePatterns[i]:SetCursorPosition(0); if state.mrp then NotePatterns[i]:Disable() end; end;
      -- disable anything that needs disabling
      self.widgets.ParseAdultGender:SetEnabled(Config.get("PARSE_GENDER"))
      self.widgets.ShowChangesMessage:SetEnable(Config.get("LOGIN_MESSAGE"))
    end;

  GeneralPanel.default =
    function(self)
      for _, widget in pairs(self.widgets) do RPTAGS.utils.config.reset(widget.setting) end;
      notify("OPTIONS_ARE_RESET");
    end;

  -- colors panel ----------------------------------------------------------------------------------------------------------------
        ColorsPanel.name = loc("CONFIG_MENU_COLORS");
        ColorsPanel.parent = OptionsPanel.name;
        InterfaceOptions_AddCategory(ColorsPanel);
        ColorsPanel.widgets = {};

  local ColorsTitle = ColorsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
        ColorsTitle:SetPoint("TOPLEFT", 16, -16);
        ColorsTitle:SetJustifyH("LEFT");
        ColorsTitle:SetText(loc("CONFIG_PAGE_COLORS"));
        ColorsPanel.title = ColorsTitle;
        ColorsPanel.version = ColorsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        ColorsPanel.version:SetPoint("TOPRIGHT", -16, -16);
        ColorsPanel.version:SetJustifyH("RIGHT");
        ColorsPanel.version:SetText(version);
  local ColorsInstructions = ColorsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        ColorsInstructions:SetPoint("TOPLEFT", ColorsTitle, "BOTTOMLEFT", 0, -16)
        ColorsInstructions:SetWidth(575);
        ColorsInstructions:SetText(hi(loc("CONFIG_DESC_COLORS")));
        ColorsInstructions:SetWordWrap(true);
        ColorsInstructions:SetJustifyH("LEFT");
        ColorsInstructions:SetJustifyV("TOP");

  lastElement = ColorsInstructions;
  xOffset = 150;
  yOffset = 12;

  local colorRefresh = true;
  for _, setting in ipairs(RPTAGS.const.CONFIG.GROUPS.COLORS)
  do local picker=PC_Color:New(ColorsPanel, loc("CONFIG_" .. setting), hi(loc("CONFIG_" .. setting .. "_TT")));
     picker.setting = setting;
     picker:SetPoint("TOPLEFT", lastElement, "BOTTOMLEFT", xOffset, -1 * yOffset);
     lastElement = picker;
     xOffset = 0; yOffset = 0;
     ColorsPanel.widgets[setting] = picker;
     picker.OnValueChanged = 
         function(self, rr, gg, bb) rr = math.floor(rr*255); gg = math.floor(gg*255); bb = math.floor(bb*255);
           local color = RPTAGS.utils.color.colorCode(rr, gg, bb); color = color:gsub("^|cff", "");
           Config.set(self.setting, color); 
           return colorRefresh and refreshAll();
         end; -- function
  end; -- for

  ColorsPanel.widgets.COLOR_RPUF.OnValueChanged = function(self, rr, gg, bb)
  local color = RPTAGS.utils.color.colorCode( math.floor(rr*255), math.floor(gg*255), math.floor(bb*255)):gsub("^|cff", "");
  Config.set(self.setting, color) setBgs(Config.get("RPUFALPHA"), color); end;
  ColorsPanel.widgets.COLOR_RPUF_TEXT.OnValueChanged = function(self, rr, gg, bb)
  local color = RPTAGS.utils.color.colorCode( math.floor(rr*255), math.floor(gg*255), math.floor(bb*255)):gsub("^|cff", "");
  Config.set(self.setting, color) RPTAGS.utils.rpuf.allFrames.textcolor(); end;
  ColorsPanel.widgets.COLOR_RPUF_TOOLTIP.OnValueChanged = function(self, rr, gg, bb)
  local color = RPTAGS.utils.color.colorCode( math.floor(rr*255), math.floor(gg*255), math.floor(bb*255)):gsub("^|cff", "");
  Config.set(self.setting, color) RPTAGS.utils.rpuf.allFrames.textcolor("tt"); end;

        -- some labels for the above
        ColorsPanel.COLORS_STATUS = ColorsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        ColorsPanel.COLORS_STATUS:SetPoint("RIGHT", ColorsPanel.widgets.COLOR_IC, "LEFT", 0, 0);
        ColorsPanel.COLORS_STATUS:SetWidth(150);
        ColorsPanel.COLORS_STATUS:SetJustifyH("LEFT");
        ColorsPanel.COLORS_STATUS:SetText(loc("CONFIG_H1_COLORS_STATUS"));
        ColorsPanel.COLORS_GENDER = ColorsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        ColorsPanel.COLORS_GENDER:SetPoint("RIGHT", ColorsPanel.widgets.COLOR_MALE, "LEFT", 0, 0);
        ColorsPanel.COLORS_GENDER:SetWidth(150);
        ColorsPanel.COLORS_GENDER:SetJustifyH("LEFT");
        ColorsPanel.COLORS_GENDER:SetText(loc("CONFIG_H1_COLORS_GENDER"));
        ColorsPanel.COLORS_COMPARISON = ColorsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        ColorsPanel.COLORS_COMPARISON:SetPoint("RIGHT", ColorsPanel.widgets.COLOR_LESSTHAN, "LEFT", 0, 0);
        ColorsPanel.COLORS_COMPARISON:SetWidth(150);
        ColorsPanel.COLORS_COMPARISON:SetJustifyH("LEFT");
        ColorsPanel.COLORS_COMPARISON:SetText(loc("CONFIG_H1_COLORS_COMPARISON"));
        ColorsPanel.COLORS_HILITE = ColorsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        ColorsPanel.COLORS_HILITE:SetPoint("RIGHT", ColorsPanel.widgets.COLOR_HILITE_1, "LEFT", 0, 0);
        ColorsPanel.COLORS_HILITE:SetWidth(150);
        ColorsPanel.COLORS_HILITE:SetJustifyH("LEFT");
        ColorsPanel.COLORS_HILITE:SetText(loc("CONFIG_H1_COLORS_HILITE"));
        ColorsPanel.COLORS_RPUF_COLORS = ColorsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        ColorsPanel.COLORS_RPUF_COLORS:SetPoint("RIGHT", ColorsPanel.widgets.COLOR_RPUF, "LEFT", 0, 0);
        ColorsPanel.COLORS_RPUF_COLORS:SetWidth(150);
        ColorsPanel.COLORS_RPUF_COLORS:SetJustifyH("LEFT");
        ColorsPanel.COLORS_RPUF_COLORS:SetText(hi(loc("RPUF_NAME")));
       
  ColorsPanel.default =
    function(self)
      colorRefresh = false;
      for _, widget in pairs(self.widgets) do RPTAGS.utils.config.reset(widget.setting) end; 
      refreshAll();
      colorRefresh = true;
      notify("COLORS_ARE_RESET");
    end;

  ColorsPanel.refresh =
    function(self)
      colorRefresh = false;
      for _, widget in pairs(self.widgets) 
      do r, g, b = RPTAGS.utils.color.hexaToNumber(Config.get(widget.setting)); widget:SetValue(r / 255, g / 255, b / 255, 1); end;
      self.widgets.COLOR_THEY:SetEnabled(Config.get("PARSE_GENDER"));
      colorRefresh = true;
    end;

  -- RPUF general panel -----------------------------------------------------------------------------------------------------------
        UnitFramesPanel.name = loc("CONFIG_MENU_RPUF_MAIN");
        UnitFramesPanel.parent = OptionsPanel.name;
        InterfaceOptions_AddCategory(UnitFramesPanel);
        UnitFramesPanel.widgets = {};
  local UnitFramesTitle = UnitFramesPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
        UnitFramesTitle:SetPoint("TOPLEFT", 16, -16);
        UnitFramesTitle:SetJustifyH("LEFT");
        UnitFramesTitle:SetText(loc("CONFIG_PAGE_RPUF_MAIN"));
        UnitFramesPanel.title = UnitFramesTitle;
        UnitFramesPanel.version = UnitFramesPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        UnitFramesPanel.version:SetPoint("TOPRIGHT", -16, -16);
        UnitFramesPanel.version:SetJustifyH("RIGHT");
        UnitFramesPanel.version:SetText(version);
  local UnitFramesInstructions = UnitFramesPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        UnitFramesInstructions:SetPoint("TOPLEFT", UnitFramesTitle, "BOTTOMLEFT", 0, -16)
        UnitFramesInstructions:SetWidth(575);
        UnitFramesInstructions:SetText(hi(loc("CONFIG_DESC_RPUF_MAIN")));
        UnitFramesInstructions:SetWordWrap(true);
        UnitFramesInstructions:SetJustifyH("LEFT");
        UnitFramesInstructions:SetJustifyV("TOP");
  local DisableRPUF = PC_Check:New(UnitFramesPanel, loc("CONFIG_DISABLE_RPUF"), hi(loc("CONFIG_DISABLE_RPUF_TT")));
        DisableRPUF:SetPoint("TOPLEFT", UnitFramesInstructions, "BOTTOMLEFT", 0, -16);
        DisableRPUF.setting = "DISABLE_RPUF";
        DisableRPUF.Text:SetWidth(160);
        DisableRPUF.neverDisable = true;
        UnitFramesPanel.widgets.DisableRPUF = DisableRPUF;
        DisableRPUF.OnValueChanged = function(self, value) Config.set(self.setting, value); disableFrames() end;
  local DisableBlizz = PC_Check:New(UnitFramesPanel, loc("CONFIG_DISABLE_BLIZZARD"), hi(loc("CONFIG_DISABLE_BLIZZARD_TT")));
        DisableBlizz:SetPoint("LEFT", DisableRPUF.Text, "RIGHT", 5, 0);
        DisableBlizz.setting = "DISABLE_BLIZZARD";
        DisableBlizz.Text:SetWidth(200); -- the text is a little long on this one
        DisableBlizz.setting = "DISABLE_BLIZZARD";
        DisableBlizz.doNotReset = true;
        UnitFramesPanel.widgets.DisableBlizz = DisableBlizz;
        DisableBlizz.OnValueChanged = function(self, value) StaticPopup_Show("RPTAGS_SAVE_AND_RELOAD") end;
  local HideInCombat = PC_Check:New(UnitFramesPanel, loc("CONFIG_RPUF_HIDE_COMBAT"), hi(loc("CONFIG_RPUF_HIDE_COMBAT_TT")));
        HideInCombat:SetPoint("TOPLEFT", DisableRPUF, "BOTTOMLEFT", 0, 0);
        HideInCombat.setting = "RPUF_HIDE_COMBAT";
        HideInCombat.Text:SetWidth(160);
        HideInCombat.OnValueChanged = function(self, value) Config.set(self.setting, value); setHide(); end;
        UnitFramesPanel.widgets.HideInCombat = HideInCombat;
  local HideInPetBattle = PC_Check:New(UnitFramesPanel, loc("CONFIG_RPUF_HIDE_PETBATTLE"), hi(loc("CONFIG_RPUF_HIDE_PETBATTLE_TT")));
        HideInPetBattle:SetPoint("LEFT", HideInCombat.Text, "RIGHT", 5, 0);
        HideInPetBattle.setting = "RPUF_HIDE_PETBATTLE";
        HideInPetBattle.Text:SetWidth(160);
        HideInPetBattle.OnValueChanged = function(self, value) Config.set(self.setting, value); setHide(); end;
        UnitFramesPanel.widgets.HideInPetBattle = HideInPetBattle;
  local HideInVehicles = PC_Check:New(UnitFramesPanel, loc("CONFIG_RPUF_HIDE_VEHICLE"), hi(loc("CONFIG_RPUF_HIDE_VEHICLE_TT")));
        HideInVehicles:SetPoint("LEFT", HideInPetBattle.Text, "RIGHT", 5, 0);
        HideInVehicles.setting = "RPUF_HIDE_VEHICLE";
        HideInVehicles.Text:SetWidth(160);
        HideInVehicles.OnValueChanged = function(self, value) Config.set(self.setting, value); setHide(); end;
        UnitFramesPanel.widgets.HideInVehicles = HideInVehicles;
  local HideInParty = PC_Check:New(UnitFramesPanel, loc("CONFIG_RPUF_HIDE_PARTY"), hi(loc("CONFIG_RPUF_HIDE_PARTY_TT")));
        HideInParty:SetPoint("TOPLEFT", HideInCombat, "BOTTOMLEFT", 0, 0);
        HideInParty.setting = "RPUF_HIDE_PARTY";
        HideInParty.Text:SetWidth(160);
        HideInParty.OnValueChanged = function(self, value) Config.set(self.setting, value); setHide(); end;
        UnitFramesPanel.widgets.HideInParty = HideInParty;
  local HideInRaid = PC_Check:New(UnitFramesPanel, loc("CONFIG_RPUF_HIDE_RAID"), hi(loc("CONFIG_RPUF_HIDE_RAID_TT")));
        HideInRaid:SetPoint("LEFT", HideInParty.Text, "RIGHT", 5, 0);
        HideInRaid.setting = "RPUF_HIDE_RAID";
        HideInRaid.Text:SetWidth(160);
        HideInRaid.OnValueChanged = function(self, value) Config.set(self.setting, value); setHide(); end;
        UnitFramesPanel.widgets.HideInRaid = HideInRaid;
  local HideWhenDead = PC_Check:New(UnitFramesPanel, loc("CONFIG_RPUF_HIDE_DEAD"), hi(loc("CONFIG_RPUF_HIDE_DEAD_TT")));
        HideWhenDead:SetPoint("LEFT", HideInRaid.Text, "RIGHT", 5, 0);
        HideWhenDead.setting = "RPUF_HIDE_DEAD";
        HideWhenDead.Text:SetWidth(160);
        HideWhenDead.OnValueChanged = function(self, value) Config.set(self.setting, value); setHide(); end;
        UnitFramesPanel.widgets.HideWhenDead = HideWhenDead; -- ##
  local LockFrames = PC_Check:New(UnitFramesPanel, loc("CONFIG_LOCK_FRAMES"), hi(loc("CONFIG_LOCK_FRAMES_TT")));
        LockFrames:SetPoint("TOPLEFT", HideInParty, "BOTTOMLEFT", 0, -16);
        LockFrames.setting = "LOCK_FRAMES";
        LockFrames.Text:SetWidth(160);
        LockFrames.OnValueChanged = function(self, value) Config.set(self.setting, value); lockFrames(); end;
        UnitFramesPanel.widgets.LockFrames = LockFrames;
  local ResetFrameLocations = CreateFrame("Button", nil, UnitFramesPanel, "UIPanelButtonTemplate");
        ResetFrameLocations:SetPoint("LEFT", LockFrames.Text, "RIGHT", 5, 0);
        ResetFrameLocations:SetWidth(185);
        ResetFrameLocations:SetText("Reset Frame Locations");
        ResetFrameLocations:SetScript("OnClick", function() RPTAGS.utils.rpuf.allFrames.resetloc(); notify("FRAME_LOCATIONS_ARE_RESET"); end);
        UnitFramesPanel.widgets.ResetFrameLocations = ResetFrameLocations;
  local PlayerFrameScale = PC_Slider:New(UnitFramesPanel, loc("CONFIG_PLAYERFRAME_SCALE"), hi(loc("CONFIG_PLAYERFRAME_SCALE_TT")), 0.25, 2, 0.05, true, true)
        PlayerFrameScale:SetPoint("TOPLEFT", LockFrames, "BOTTOMLEFT", 0, -16);
        PlayerFrameScale.setting = "PLAYERFRAME_SCALE";
        PlayerFrameScale:SetWidth(180);
        PlayerFrameScale.OnValueChanged = function(self, value) Config.set(self.setting, value); RPUF_Player:SetScale(value); RPUF_Player:UpdateAllElements('now'); end; -- function
        PlayerFrameScale:SetValue(Config.get("PLAYERFRAME_SCALE"));
        UnitFramesPanel.widgets.PlayerFrameScale = PlayerFrameScale;
  local TargetFrameScale = PC_Slider:New(UnitFramesPanel, loc("CONFIG_TARGETFRAME_SCALE"), hi(loc("CONFIG_TARGETFRAME_SCALE_TT")), 0.25, 2, 0.05, true, true)
        TargetFrameScale:SetPoint("LEFT", PlayerFrameScale, "RIGHT", 15, 0)
        TargetFrameScale:SetWidth(180);
        TargetFrameScale.setting = "TARGETFRAME_SCALE";
        TargetFrameScale.OnValueChanged = function(self, value) Config.set(self.setting, value); RPUF_Target:SetScale(value); RPUF_Target:UpdateAllElements('now'); end; -- function
        TargetFrameScale:SetValue(Config.get("TARGETFRAME_SCALE"));
        UnitFramesPanel.widgets.TargetFrameScale = TargetFrameScale;
  local FocusFrameScale = PC_Slider:New(UnitFramesPanel, loc("CONFIG_FOCUSFRAME_SCALE"), hi(loc("CONFIG_FOCUSFRAME_SCALE_TT")), 0.25, 2, 0.05, true, true)
        FocusFrameScale:SetPoint("LEFT", TargetFrameScale, "RIGHT", 15, 0);
        FocusFrameScale:SetWidth(180);
        FocusFrameScale.setting = "FOCUSFRAME_SCALE";
        FocusFrameScale.OnValueChanged = function(self, value) Config.set(self.setting, value); RPUF_Focus:SetScale(value); RPUF_Focus:UpdateAllElements('now'); end; -- function
        FocusFrameScale:SetValue(Config.get("FOCUSFRAME_SCALE"));
        UnitFramesPanel.widgets.FocusFrameScale = FocusFrameScale;

  UnitFramesPanel.disableRPUF = 
    function(self, disable) for _, widget in pairs(self.widgets) do widget:SetEnabled(not disable or widget.neverDisable) end; end;

  UnitFramesPanel.refresh = 
    function(self) for _, widget in pairs(self.widgets) do if widget.setting then widget:SetValue(Config.get(widget.setting)); end; end; end;

  UnitFramesPanel.default =
    function(self)
      for _, widget in pairs(self.widgets) do if widget.setting and not widget.doNotReset then Config.reset(widget.setting) end; end;
      scaleFrames(true); lockFrames(true); setHide(true);
      notify("OPTIONS_ARE_RESET");
      if state.elvui then DisableBlizz:Disable() end;
    end;

  -- RPUF layout panel ------------------------------------------------------------------------------------------------------------
        LayoutPanel.name = loc("CONFIG_MENU_RPUF_LAYOUT")
        LayoutPanel.parent = OptionsPanel.name;
        InterfaceOptions_AddCategory(LayoutPanel);
        LayoutPanel.widgets = {};
  local LayoutTitle = LayoutPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
        LayoutTitle:SetPoint("TOPLEFT", LayoutPanel, "TOPLEFT", 16, -16);
        LayoutTitle:SetJustifyH("LEFT");
        LayoutTitle:SetText(loc("CONFIG_PAGE_RPUF_LAYOUT"));
        LayoutPanel.title = LayoutTitle;
        LayoutPanel.version = LayoutPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        LayoutPanel.version:SetPoint("TOPRIGHT", LayoutPanel, "TOPRIGHT", -16, -16);
        LayoutPanel.version:SetJustifyH("RIGHT");
        LayoutPanel.version:SetText(version);
  local LayoutInstructions = LayoutPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        LayoutInstructions:SetPoint("TOPLEFT", LayoutTitle, "BOTTOMLEFT", 0, -16)
        LayoutInstructions:SetWidth(575);
        LayoutInstructions:SetText(hi(loc("CONFIG_DESC_RPUF_LAYOUT")));
        LayoutInstructions:SetWordWrap(true);
        LayoutInstructions:SetJustifyH("LEFT");
        LayoutInstructions:SetJustifyV("TOP");
        LayoutPanel.instructions = LayoutInstructions;
  local UnitFrameMockup = CreateFrame("Frame", nil, LayoutPanel);
        UnitFrameMockup:SetBackdrop(RPTAGS.const.BACKDROP[Config.get("RPUF_BACKDROP")]);
        UnitFrameMockup:SetBackdropColor(0.5, 0.5, 0.5, 0.15);
        UnitFrameMockup:SetSize(590, 325);
        UnitFrameMockup:SetPoint("TOPLEFT", LayoutInstructions, "BOTTOMLEFT", 0, -48);
  local IconStandin = UnitFrameMockup:CreateTexture(nil, "ARTWORK");
        IconStandin:SetTexture("INTERFACE\\AddOns\\RP_Tags\\resources\\Graphics\\rptags-logo");
        IconStandin:SetPoint("TOPLEFT", UnitFrameMockup, "TOPLEFT", 36, -16);
        IconStandin:SetWidth(60);
        IconStandin:SetHeight(60);
  local PortraitStandin = UnitFrameMockup:CreateTexture(nil, "ARTWORK")
        PortraitStandin:SetPoint("TOPLEFT", UnitFrameMockup, "TOPLEFT", 125, -16);
        PortraitStandin:SetWidth(150);
        PortraitStandin:SetHeight(170);
  local IconWidthSlider = PC_Slider:New(LayoutPanel, loc("CONFIG_ICONWIDTH"), hi(loc("CONFIG_ICONWIDTH_TT")), 10, 75, 1, false, true)
        IconWidthSlider:SetWidth(100);
        IconWidthSlider:SetPoint("TOPLEFT", UnitFrameMockup, "TOPLEFT", 16, -80);
        IconWidthSlider.setting = "ICONWIDTH";
        IconWidthSlider.OnValueChanged = function(self, value) Config.set(self.setting, value); resizeAll() end;
        LayoutPanel.widgets.IconWidthSlider = IconWidthSlider;
--  local PortraitStyle = PC_Dropdown:New(UnitFrameMockup, loc("CONFIG_PORTRAIT_STYLE"), loc("CONFIG_PORTRAIT_STYLE_TT"), valuesToMenu("PORTRAIT_STYLE"));
--        PortraitStyle:SetWidth(150);
--        PortraitStyle:SetPoint("LEFT", PortraitStandin, "LEFT", 0, 0);
--        PortraitStyle:SetPoint("BOTTOM", UnitFrameMockup, "BOTTOM", 0, 16);
--        PortraitStyle.setting = "PORTRAIT_STYLE";
--        PortraitStyle.OnValueChanged = 
--          function(self, value) 
--            Config.set(self.setting, value); 
--            if value == "FACE" then PortraitStandin:SetTexture("Interface\\Pictures\\obj_nesingwary_256")
--            else PortraitStandin:SetTexture("INTERFACE\\Pictures\\Linken__Color_256px"); end;
--            RPTAGS.utils.rpuf.allFrames.portraitStyle() end;
--        LayoutPanel.widgets.PortraitStyle = PortraitStyle; -- ##
  local PortraitWidthSlider = PC_Slider:New(LayoutPanel, loc("CONFIG_PORTWIDTH"), hi(loc("CONFIG_PORTWIDTH_TT")), 25, 200, 5, false, true)
        PortraitWidthSlider:SetWidth(150);
        PortraitWidthSlider:SetPoint("TOP", PortraitStandin, "BOTTOM", 0, 0);
        PortraitWidthSlider.setting = "PORTWIDTH";
        PortraitWidthSlider.OnValueChanged = function(self, value) Config.set(self.setting, value); resizeAll() end;
        LayoutPanel.widgets.PortraitWidthSlider = PortraitWidthSlider;
  local InfoWidthSlider = PC_Slider:New(LayoutPanel, loc("CONFIG_INFOWIDTH"), hi(loc("CONFIG_INFOWIDTH_TT")), 100, 400, 10, false, true)
        InfoWidthSlider:SetWidth(290);
        InfoWidthSlider:SetPoint("TOPRIGHT", UnitFrameMockup, "TOPRIGHT", -16, -16);
        InfoWidthSlider.setting = "INFOWIDTH";
        InfoWidthSlider.OnValueChanged = function(self, value) Config.set(self.setting, value); resizeAll() end;
        LayoutPanel.widgets.InfoWidthSlider = InfoWidthSlider;
  local DetailsHeightSlider = PC_Slider:New(LayoutPanel, loc("CONFIG_DETAILHEIGHT"), hi(loc("CONFIG_DETAILHEIGHT_TT")), 20, 250, 5, false, true)
        DetailsHeightSlider:SetWidth(200);
        DetailsHeightSlider:SetPoint("TOPRIGHT", InfoWidthSlider, "BOTTOMRIGHT", 0, -16);
        DetailsHeightSlider.setting = "DETAILHEIGHT";
        DetailsHeightSlider.OnValueChanged = function(self, value) Config.set(self.setting, value); resizeAll() end;
        LayoutPanel.widgets.DetailsHeightSlider = DetailsHeightSlider;

  local StatusAlignment = PC_Dropdown:New(UnitFrameMockup, loc("CONFIG_STATUS_ALIGN"), loc("CONFIG_STATUS_ALIGN_TT"), valuesToMenu("STATUS_ALIGN"));
        StatusAlignment:SetPoint("BOTTOMRIGHT", UnitFrameMockup, "BOTTOMRIGHT", -16, 16);
        StatusAlignment.setting = "STATUS_ALIGN";
        StatusAlignment:SetWidth(140);
        StatusAlignment.OnValueChanged = function(self, value) Config.set(self.setting, value) RPTAGS.utils.rpuf.allFrames.statusAlign(value); end;
        LayoutPanel.widgets.StatusAlignment = StatusAlignment;

  local StatusHeightSlider = PC_Slider:New(LayoutPanel, loc("CONFIG_STATUSHEIGHT"), hi(loc("CONFIG_STATUSHEIGHT_TT")), 15, 140, 5, false, true)
        StatusHeightSlider:SetPoint("BOTTOM", StatusAlignment, "TOP", 0, 0);
        StatusHeightSlider.setting = "STATUSHEIGHT";
        StatusHeightSlider:SetWidth(140);
        StatusHeightSlider.OnValueChanged = function(self, value) Config.set(self.setting, value); resizeAll() end;
        LayoutPanel.widgets.StatusHeightSlider = StatusHeightSlider; 

  local StatusTexture = PC_Dropdown:New(UnitFrameMockup, loc("CONFIG_STATUS_TEXTURE"), loc("CONFIG_STATUS_TEXTURE_TT"), valuesToMenu("STATUS_TEXTURE"));
        StatusTexture:SetPoint("RIGHT", StatusAlignment, "LEFT", -5, 0);
        StatusTexture.setting = "STATUS_TEXTURE";
        StatusTexture:SetWidth(140);
        StatusTexture.OnValueChanged = function(self, value) Config.set(self.setting, value) RPTAGS.utils.rpuf.allFrames.statusTexture(value) end;
        LayoutPanel.widgets.StatusTexture = StatusTexture;

  local StatusLabel = UnitFrameMockup:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
        StatusLabel:SetText(loc("STATUS_BAR"));
        StatusLabel:SetWidth(140);
        StatusLabel:SetJustifyH("LEFT");
        StatusLabel:SetPoint("RIGHT", StatusHeightSlider, "LEFT", 0, 0);

  local GapSizeSlider = PC_Slider:New(LayoutPanel, loc("CONFIG_GAPSIZE"), hi(loc("CONFIG_GAPSIZE_TT")), 0, 20, 1, false, true)
        GapSizeSlider.setting = "GAPSIZE";
        GapSizeSlider:SetWidth(100);
        GapSizeSlider.labelText:ClearAllPoints();
        GapSizeSlider.labelText:SetJustifyH("RIGHT");
        GapSizeSlider.labelText:SetPoint("RIGHT", GapSizeSlider, "LEFT", -8, 0);
        GapSizeSlider.OnValueChanged = function(self, value) Config.set(self.setting, value); resizeAll() end;
        GapSizeSlider:SetPoint("BOTTOMRIGHT", UnitFrameMockup, "TOPRIGHT", 0, 0);
        LayoutPanel.widgets.GapSizeSlider = GapSizeSlider;
  local FrameBorder = PC_Dropdown:New(UnitFrameMockup, loc(""), loc("CONFIG_RPUF_BACKDROP_TT"), valuesToMenu("RPUF_BACKDROP"));
        FrameBorder:SetPoint("BOTTOMLEFT", UnitFrameMockup, "TOPLEFT", 0, 0);
        FrameBorder:SetWidth(185);
        FrameBorder.setting = "RPUF_BACKDROP";
        LayoutPanel.widgets.FrameBorder = FrameBorder;
        FrameBorder.OnValueChanged = 
          function(self, value) 
            Config.set(self.setting, value); setBackdrop(value);
            UnitFrameMockup:SetBackdrop(RPTAGS.const.BACKDROP[value]);
            UnitFrameMockup:SetBackdropColor(0.5, 0.5, 0.5, 0.15);
          end;
  local AlphaSlider = PC_Slider:New(LayoutPanel, loc("CONFIG_RPUFALPHA"), hi(loc("CONFIG_RPUFALPHA_TT")), 0, 1, 0.05, true, true);
        AlphaSlider.setting = "RPUFALPHA";
        AlphaSlider.OnValueChanged = function(self, value) Config.set(self.setting, value * 100); setBgs(value * 100, Config.get("COLOR_RPUF")); end;
        AlphaSlider:SetWidth(200);
        AlphaSlider.refreshAsPercent = true;
        AlphaSlider:SetPoint("BOTTOMLEFT", UnitFrameMockup, "BOTTOMLEFT", 16, 16);
        LayoutPanel.widgets.AlphaSlider = AlphaSlider;
  local PlayerLayout = PC_Dropdown:New(LayoutPanel, loc("CONFIG_PLAYERLAYOUT"), loc("CONFIG_PLAYERLAYOUT_TT"), valuesToMenu("PLAYERLAYOUT"));
        PlayerLayout:SetPoint("TOPLEFT", UnitFrameMockup, "BOTTOMLEFT", 0, -16);
        PlayerLayout:SetWidth(185);
        PlayerLayout.setting = "PLAYERLAYOUT";
        PlayerLayout.OnValueChanged = 
          function(self, value) 
            Config.set(self.setting, value); 
            resizeFrame(RPUF_Player); 
            RPTAGS.utils.rpuf.portraitStyle(RPUF_Player);
            setHide(true); 
          end;
        LayoutPanel.widgets.PlayerLayout = PlayerLayout;
  local TargetLayout = PC_Dropdown:New(LayoutPanel, loc("CONFIG_TARGETLAYOUT"), loc("CONFIG_TARGETLAYOUT_TT"), valuesToMenu("TARGETLAYOUT"));
        TargetLayout.setting = "TARGETLAYOUT";
        TargetLayout:SetPoint("LEFT", PlayerLayout, "RIGHT", 16, 0);
        TargetLayout:SetWidth(185);
        TargetLayout.OnValueChanged = 
          function(self, value) 
            Config.set(self.setting, value); 
            resizeFrame(RPUF_Target); 
            RPTAGS.utils.rpuf.portraitStyle(RPUF_Target); 
            setHide(true); 
          end;
        LayoutPanel.widgets.TargetLayout = TargetLayout;
  local FocusLayout = PC_Dropdown:New(LayoutPanel, loc("CONFIG_FOCUSLAYOUT"), loc("CONFIG_FOCUSLAYOUT_TT"), valuesToMenu("FOCUSLAYOUT"))
        FocusLayout.setting = "FOCUSLAYOUT";
        FocusLayout:SetPoint("LEFT", TargetLayout, "RIGHT", 16, 0);
        FocusLayout:SetWidth(185);
        FocusLayout.OnValueChanged = 
          function(self, value) 
            Config.set(self.setting, value); 
            resizeFrame(RPUF_Focus); 
            RPTAGS.utils.rpuf.portraitStyle(RPUF_Focus); 
            setHide(true); 
          end;
        LayoutPanel.widgets.FocusLayout = FocusLayout;

--  local TargetTargetLayout = PC_Dropdown:New(LayoutPanel, loc("CONFIG_TARGETTARGETLAYOUT"), loc("CONFIG_TARGETTARGETLAYOUT_TT"), valuesToMenu("TARGETTARGETLAYOUT"))
--        TargetTargetLayout.setting = "TARGETTARGETLAYOUT";
--        TargetTargetLayout:SetPoint("TOPLEFT", PlayerLayout, "BOTTOMLEFT", 0, 0)
--        TargetTargetLayout:SetWidth(185);
--        TargetTargetLayout.OnValueChanged = function(self, value) Config.set(self.setting, value); resizeFrame(RPUF_TargetTarget); setHide(true); end;
--        LayoutPanel.widgets.TargetTargetLayout = TargetTargetLayout;

--   local PartyLayout = PC_Dropdown:New(LayoutPanel, loc("CONFIG_PARTYLAYOUT"), loc("CONFIG_PARTYLAYOUT_TT"), valuesToMenu("PARTYLAYOUT"))
--         PartyLayout.setting = "PARTYLAYOUT";
--         PartyLayout:SetPoint("LEFT", TargetTargetLayout, "RIGHT", 16, 0);
--         PartyLayout:SetWidth(185);
--     --    PartyLayout.OnValueChanged = function(self, value) Config.set(self.setting, value); resizeFrame(RPUF_Party); setHide(true); end;
--         LayoutPanel.widgets.PartyLayout = PartyLayout;
--         PartyLayout.disable = true;
--
--   local RaidLayout = PC_Dropdown:New(LayoutPanel, loc("CONFIG_RAIDLAYOUT"), loc("CONFIG_RAIDLAYOUT_TT"), valuesToMenu("RAIDLAYOUT"))
--         RaidLayout.setting = "RAIDLAYOUT";
--         RaidLayout:SetPoint("LEFT", PartyLayout, "RIGHT", 16, 0);
--         RaidLayout:SetWidth(185);
--     --    RaidLayout.OnValueChanged = function(self, value) Config.set(self.setting, value); resizeFrame(RPUF_Raid); setHide(true); end;
--         LayoutPanel.widgets.RaidLayout = RaidLayout;
--         RaidLayout.disable = true;
--
--   local PartyOrientation = PC_Dropdown:New(LayoutPanel, loc("CONFIG_PARTY_ORIENTATION"), loc("CONFIG_PARTY_ORIENTATION_TT"), valuesToMenu("PARTY_ORIENTATION"))
--         PartyOrientation.setting = "PARTY_ORIENTATION";
--         PartyOrientation:SetPoint("TOPLEFT", PartyLayout, "BOTTOMLEFT", 0, 0);
--         PartyOrientation:SetWidth(185);
--        -- PartyOrientation.OnValueChanged = function(self, value) Config.set(self.setting, value); resizeFrame(RPUF_PartyOrientation); setHide(true); end;
--         LayoutPanel.widgets.PartyOrientation = PartyOrientation;
--         PartyOrientation.disable = true;
--
--   local RaidGrid = PC_Dropdown:New(LayoutPanel, loc("CONFIG_RAID_GRID"), loc("CONFIG_RAID_GRID_TT"), valuesToMenu("RAID_GRID"))
--         RaidGrid.setting = "RAID_GRID";
--         RaidGrid:SetPoint("LEFT", PartyOrientation, "RIGHT", 16, 0);
--         RaidGrid:SetWidth(185);
--        -- RaidGrid.OnValueChanged = function(self, value) Config.set(self.setting, value); resizeFrame(RPUF_OtherRaid); setHide(true); end;
--         LayoutPanel.widgets.RaidGrid = RaidGrid;
--         RaidGrid.disable = true;
--  
  LayoutPanel.disableRPUF =
    function(self, disable)
      for _, widget in pairs(self.widgets) do if disable then widget:Disable() else widget:Enable() end; end;
      IconStandin:SetDesaturated(disable);
      PortraitStandin:SetDesaturated(disable);
      if disable then self.instructions:SetText(hi(loc("WARNING_RPUF_DISABLED"))); else self.instructions:SetText(hi(loc("CONFIG_DESC_RPUF_LAYOUT"))); end;
    end;

  LayoutPanel.default = function(self) for _, widget in pairs(self.widgets) do Config.reset(widget.setting) end; resizeAll(); notify("FRAMES_ARE_RESET"); end;

  LayoutPanel.refresh =
    function(self)
    for _, widget in pairs(self.widgets)
    do if widget.refreshAsPercent then widget:SetValue(Config.get(widget.setting) / 100)
       else widget:SetValue(Config.get(widget.setting));
       end; -- if
       if widget.disable then widget:Disable(); end;
    end; -- for

    if Config.get("PORTRAIT_STYLE") == "FACE"  -- change the portrait standing depending on the setting
    then PortraitStandin:SetTexture("Interface\\Pictures\\obj_nesingwary_256")
    else PortraitStandin:SetTexture("INTERFACE\\Pictures\\Linken__Color_256px");
    end

  end;

  -- ##
  -- RPUF tag panels panel --------------------------------------------------------------------------------------------------------
  TagsPanel.name = loc("CONFIG_MENU_RPUF_PANELS");
  TagsPanel.parent = OptionsPanel.name;
  InterfaceOptions_AddCategory(TagsPanel);
  TagsPanel.widgets = {};
  TagsPanel.tagviews = {};

  local TagsTitle = TagsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
  TagsTitle:SetPoint("TOPLEFT", 16, -16);
  TagsTitle:SetJustifyH("LEFT");
  TagsTitle:SetText(loc("CONFIG_PAGE_RPUF_PANELS"));
  TagsPanel.title = TagsTitle;
  
  TagsPanel.version = TagsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
  TagsPanel.version:SetPoint("TOPRIGHT", -16, -16);
  TagsPanel.version:SetJustifyH("RIGHT");
  TagsPanel.version:SetText(version);

  local TagsInstructions = TagsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
  TagsInstructions:SetPoint("TOPLEFT", TagsTitle, "BOTTOMLEFT", 0, -16)
  TagsInstructions:SetWidth(575);
  TagsInstructions:SetText(hi(loc("CONFIG_DESC_RPUF_PANELS")));
  TagsInstructions:SetWordWrap(true);
  TagsInstructions:SetJustifyH("LEFT");
  TagsInstructions:SetJustifyV("TOP");
  TagsPanel.instructions = TagsInstructions;

  TagsPanel.panelColumnLabel = TagsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
  TagsPanel.panelColumnLabel:SetWidth(130);
  TagsPanel.panelColumnLabel:SetPoint("TOPLEFT", TagsInstructions, "BOTTOMLEFT", 0, -16);
  TagsPanel.panelColumnLabel:SetText("Panel Name");
  TagsPanel.panelColumnLabel:SetJustifyH("LEFT");

  TagsPanel.tagsColumnLabel = TagsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
  TagsPanel.tagsColumnLabel:SetWidth(150);
  TagsPanel.tagsColumnLabel:SetPoint("LEFT", TagsPanel.panelColumnLabel, "RIGHT", 15, 0);
  TagsPanel.tagsColumnLabel:SetText("Panel Tags");
  TagsPanel.tagsColumnLabel:SetJustifyH("LEFT");

  TagsPanel.tooltipColumnLabel = TagsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
  TagsPanel.tooltipColumnLabel:SetWidth(150);
  TagsPanel.tooltipColumnLabel:SetPoint("LEFT", TagsPanel.tagsColumnLabel, "RIGHT", 5 + 50 + 25, 0);
  TagsPanel.tooltipColumnLabel:SetText("Tooltip Tags");
  TagsPanel.tooltipColumnLabel:SetJustifyH("LEFT");

  lastElement = TagsPanel.panelColumnLabel;
  xOffset = 0;
  yOffset = -8;
  local line = true;

  RPTAGS.cache.tagsRefresh = true;

  for _, paneltooltippair in ipairs({
    { "NAMEPANEL",   "NAME_TOOLTIP",     }, { "INFOPANEL",   "INFO_TOOLTIP",     }, { "STATUSPANEL", "STATUS_TOOLTIP",   },
    { "DETAILPANEL", "DETAIL_TOOLTIP",   }, { "PORTRAIT",    "PORTRAIT_TOOLTIP", }, { "ICON_1",      "ICON_1_TOOLTIP",   },
    { "ICON_2",      "ICON_2_TOOLTIP",   }, { "ICON_3",      "ICON_3_TOOLTIP",   }, { "ICON_4",      "ICON_4_TOOLTIP",   },
    { "ICON_5",      "ICON_5_TOOLTIP",   }, { "ICON_6",      "ICON_6_TOOLTIP",   }, }) 
  do local panel, tooltip = unpack(paneltooltippair)

     local panelDisplay = CreateFrame("Frame", nil, TagsPanel);
     panelDisplay:SetPoint("TOPLEFT", lastElement, "BOTTOMLEFT", xOffset, yOffset);
     panelDisplay:SetWidth(585);
     panelDisplay:SetHeight(20);

     if line then panelDisplay:SetBackdrop(RPTAGS.const.BACKDROP.ORIGINAL); panelDisplay:SetBackdropColor(0.5, 0.5, 0.5, 0.10) end;

     line = not line;

     panelDisplay.label = panelDisplay:CreateFontString(nil, "OVERLAY", "GameFontNormal");
     panelDisplay.label:SetPoint("LEFT", panelDisplay, "LEFT", 10, 0);
     panelDisplay.label:SetWidth(120);
     panelDisplay.label:SetJustifyH("LEFT");
     panelDisplay.label:SetTextColor(1, 1, 1);
     panelDisplay.label:SetText(loc("CONFIG_" .. panel));

     panelDisplay.rawText = panelDisplay:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
     panelDisplay.rawText:SetPoint("LEFT", panelDisplay.label, "RIGHT", 15, 0);
     panelDisplay.rawText:SetWidth(150);
     panelDisplay.rawText:SetJustifyH("LEFT");
     panelDisplay.rawText:SetWordWrap(false);
     panelDisplay.rawText:SetTextColor(0.7, 0.7, 0.7);
     panelDisplay.rawText.setting = panel;
     TagsPanel.tagviews[panel] = panelDisplay.rawText;

     panelDisplay.editButton = CreateFrame("Button", nil, panelDisplay, "UIPanelButtonTemplate");
     panelDisplay.editButton:SetPoint("LEFT", panelDisplay.rawText, "RIGHT", 5, 0);
     panelDisplay.editButton:SetText("Edit");
     panelDisplay.editButton:SetWidth(50);
     panelDisplay.editButton.setting = panel;
     panelDisplay.editButton:SetScript("OnClick", openEditor);
     TagsPanel.widgets[panel] = panelDisplay.editButton;

     panelDisplay.rawTooltip = panelDisplay:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
     panelDisplay.rawTooltip:SetPoint("LEFT", panelDisplay.editButton, "RIGHT", 25, 0);
     panelDisplay.rawTooltip:SetWidth(150);
     panelDisplay.rawTooltip:SetJustifyH("LEFT");
     panelDisplay.rawTooltip:SetWordWrap(false);
     panelDisplay.rawTooltip:SetTextColor(0.7, 0.7, 0.7);
     panelDisplay.rawTooltip.setting = tooltip;
     TagsPanel.tagviews[tooltip] = panelDisplay.rawTooltip;

     panelDisplay.editTooltipButton = CreateFrame("Button", nil, panelDisplay, "UIPanelButtonTemplate");
     panelDisplay.editTooltipButton:SetPoint("LEFT", panelDisplay.rawTooltip, "RIGHT", 5, 0);
     panelDisplay.editTooltipButton:SetText("Edit");
     panelDisplay.editTooltipButton:SetWidth(50);
     panelDisplay.editTooltipButton.setting = tooltip;
     panelDisplay.editTooltipButton:SetScript("OnClick", openEditor);

     TagsPanel.widgets[tooltip] = panelDisplay.editTooltipButton;

     lastElement = panelDisplay;
     xOffset = 0;
     yOffset = -4;

  end; -- for

  TagsPanel.disableRPUF =
    function(self, disable)
      for _, widget in pairs(self.widgets) do widget:SetEnabled(not disable) end;
      if   disable then self.instructions:SetText(hi(loc("WARNING_RPUF_DISABLED"))); else self.instructions:SetText(hi(loc("CONFIG_DESC_RPUF_PANELS"))); end;
    end;
  TagsPanel.default =
    function(self)
      RPTAGS.cache.tagsRefresh = false;
      for _, tagview in pairs(self.tagviews) 
      do RPTAGS.utils.config.reset(tagview.setting) end;
      notify("TAGS_ARE_RESET"); refreshAll();
      RPTAGS.cache.tagsRefresh = true;
    end;
  TagsPanel.refresh =
    function(self)
      for _, tagview in pairs(self.tagviews) do tagview:SetText(Config.get(tagview.setting)) end; 
      self.widgets.PORTRAIT:Disable();
    end;

  if state.trp3
  then 
    TRP3_API.navigation.menu.registerMenu(
        { id = "main_91_config_rptags_moved",
          text = loc("APP_NAME"),
          isChildOf = "main_90_config",
          onSelected = function()
            TRP3_MainFrame:Hide();
            TRP3_API.navigation.menu.selectMenu("main_90_config");
            InterfaceOptionsFrame_Show();
            InterfaceOptionsFrame_OpenToCategory(RPTAGS_Main_OptionsPanel)
          end,
          });

    if   RPTAGS.cache.TRP3_Config_Imported then notify("IMPORT_TRP3_CONFIG_START"); notify("IMPORT_TRP3_CONFIG_DONE"); end; -- what don't look at me like that
  end;

  StaticPopupDialogs["RPTAGS_SAVE_AND_RELOAD"] = {
    showAlert = 1,
    text = loc("RELOAD_UI_WARNING"),
    button1 = loc("RELOAD_UI_WARNING_YES"),
    button2 = loc("RELOAD_UI_WARNING_NO"),
    exclusive = true,
    OnAccept = function() Config.set("DISABLE_BLIZZARD", not(Config.get("DISABLE_BLIZZARD"))); ReloadUI() end,
    OnCancel = function() DisableBlizz:SetValue(Config.get("DISABLE_BLIZZARD")) end,
    timeout = 60,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
  }

  return "ui-config"

end); -- startup function


