local RPTAGS = RPTAGS;

table.insert(RPTAGS.startup, function(state) -- startup function
  
  local lang   = "enUS";
  local Locale = RPTAGS.locale[lang];
  
  if not RPTAGS.cache.tagorder then RPTAGS.cache.tagorder = {}; end; if not RPTAGS.cache.tagcache   then RPTAGS.cache.tagcache   = {}; end;

  Elvui = RPTAGS.utils.elvui;
  Rpuf  = RPTAGS.utils.rpuf;
  
  local function makeCache(tag)
    if not RPTAGS.cache.tagcache[tag] then RPTAGS.cache.tagcache[tag] = {} end;
    return RPTAGS.cache.tagcache[tag];
  end; -- function
  
  local TAG_DATA = RPTAGS.cache.TAG_DATA;

  local function createTag(name, aliases, label, desc, group, alias_for, label_for, size, scale)
    local SCALES = { ":xl", ":l", ":m", ":s", ":xs", };
    local data = makeCache(name);
    data.name         = name     ;
    data.label        = label    ;
    data.desc         = desc     ;
    data["alias-for"] = alias_for;
    data["label-for"] = label_for;
    data.help         = group.key;
    data.group        = group.key;
    data.scale        = scale    ;
    if group.external then data.external = true; return true end; -- createTag(name,             aliases, label, desc, group, alias_for, label_for, size, scale); end;
    if label          then                                           createTag(name .. "-label", nil    , nil  , nil , group, nil      , name     , size, nil  ); end;
    if aliases        then for _, aka   in ipairs(aliases)        do createTag(aka             , nil    , label, nil , group, name     , nil      , size, nil  ); end; end;
    if size           then for _, scale in ipairs(SCALES)         do createTag(name .. scale   , nil    , label, nil , group, nil      , nil      , nil , scale); end; end;
  end;

  -- --------------------------------------------------------------------------------------------------------------------------------------------------
  for _, tagGroup in ipairs(TAG_DATA)
  do  Locale["TAG_GROUP_" .. tagGroup.key] = tagGroup.title;
  
      table.insert(RPTAGS.cache.tagorder, {"submenu", tagGroup.title, tagGroup.help, } );
  
      for _, tagData in ipairs(tagGroup.tags)
      do  local tagName, aliases, label, desc = tagData.name, tagData.alias, tagData.label, tagData.desc;
          if   tagName
          then 
               if desc and tagData.unsup and state[tagData.unsup] then desc = "|cff999999" .. desc .. "|r" end;
               table.insert(RPTAGS.cache.tagorder, { "tag", desc, "[" .. tagName .. "]", } );
  
               if   tagGroup.external
               then local ExtagData                       = makeCache(tagName);   ExtagData.name         = tagName;        ExtagData.desc         = desc;
                          ExtagData.label                 = nil;                  ExtagData.aliases      = nil;            ExtagData["alias-for"] = nil;
                          ExtagData["label-for"]          = nil;                  ExtagData.external     = true;           ExtagData.help         = tagGroup.key;
                          ExtagData.group                 = tagGroup.key;
               else local RptagData                       = makeCache(tagName);   RptagData.name         = tagName;        RptagData.desc         = desc;
                          RptagData.label                 = label;                RptagData.aliases      = aliases;        RptagData["alias-for"] = nil;
                          RptagData["label-for"]          = nil;                  RptagData.help         = tagGroup.key;   RptagData.group        = tagGroup.key;
                          RptagData.unsup                 = tagData.unsup;
                    if   label 
                    then local LabelData                  = makeCache(tagName .. "-label");                                  
                               LabelData.name             = tagName .. "-label";  LabelData.desc         = desc;           LabelData.label        = nil;
                               LabelData.aliases          = nil;                  LabelData["alias-for"] = nil;            LabelData["label-for"] = tagName; 
                               LabelData.help             = tagGroup.key;         LabelData.group        = tagGroup.key;
                               LabelData.unsup            = tagData.unsup;
                    end; -- if label
                    if   aliases 
                    then for _, aka in ipairs(aliases)
                         do  local AliasData              = makeCache(aka);       AliasData.name         = aka;            AliasData.desc         = desc;
                                   AliasData.label        = label;                AliasData.aliases      = nil;            AliasData["alias-for"] = tagName;
                                   AliasData.help         = tagGroup.key;         AliasData.group        = tagGroup.key;   AliasData["label-for"] = nil;
                                   AliasData.unsup        = tagData.unsup;
       
                             if   label -- labels for aliases
                             then local LabelData         = makeCache(aka .. "-label");
                                        LabelData.name    = aka .. "-label";      LabelData.desc         = desc;           LabelData.label        = nil;
                                        LabelData.aliases = nil;                  LabelData["alias-for"] = nil;            LabelData["label-for"] = aka
                                        LabelData.help    = tagGroup.help;        LabelData.group        = tagGroup.key;
                                        LabelData.unsup   = tagData.unsup;
                             end; -- if label for aliases
                         end; -- for aka
                    end; --  if aliases
               end; -- if not external
          elseif 
            tagData.title then table.insert(RPTAGS.cache.tagorder, { "title", tagData.title or "", nil });
          end -- if tagName
      end; -- for tag
  
  end; -- for tagGroup

  local function testIfTagExists(t) -- oUF has seven "virtual" tags that don't show up in their tag tables
    if t=="curhp" or t=="curpp" or t=="maxhp" or t=="maxpp" or t=="class" or t=="faction" or t=="race" then return true end;
    return RPTAGS.oUF.Tags.Events[t] ~= nil;
  end; -- function
    
  local function prepareTagData(tag, method, extraEvents)
    local  TagData = makeCache(tag);
    if   not TagData["alias-for"] and not TagData["label-for"]
    then TagData.method      = method;
         TagData.extraEvents = extraEvents;
    
         if   TagData.label
         then local LabelData             = makeCache(tag .. "-label");
                    LabelData.method      = function(u) local value = method(w); if value and value ~= "" then return TagData.label..": "..value; end end; -- function
                    LabelData.extraEvents = extraEvents;
         end; --if label
    
         if   TagData.aliases
         then for _, aka in ipairs(TagData.aliases)
              do  local AliasData = makeCache(aka);
                  AliasData.method = method;
                  AliasData.extraEvents = extraEvents;
                  if   AliasData.label
                  then local  LabelData             = makeCache(aka .. "-label");
                              LabelData.method      = function(u) local value = method(u); if value and value ~= "" then return AliasData.label..": "..value; end end;
                              LabelData.extraEvents = extraEvents;
                  end; -- if alias label
              end; -- for aka
         end; -- if aliases
    end; -- if not alias-for or label-for
  end;  -- function
  
  local function registerAllTags()   if state.elvui then Elvui.tags.registerall()    end; if state.rpuf then Rpuf.tags.registerall()    end; end; -- function
  local function refreshFrame(frame) if state.elvui then Elvui.frames.refresh(frame) end; if state.rpuf then Rpuf.frame.refresh(frame); end; end; -- function      
  local function refreshAll()        if state.elvui then Elvui.frames.refreshAll();  end; if state.rpuf then Rpuf.allFrames.refresh();  end; end; -- function
    
  if not RPTAGS.utils.tags then RPTAGS.utils.tags = {} end;

  RPTAGS.utils.tags.register     = registerAllTags;
  RPTAGS.utils.tags.exists       = testIfTagExists;
  RPTAGS.utils.tags.prepare      = prepareTagData;
  RPTAGS.utils.tags.refreshFrame = refreshFrame;
  RPTAGS.utils.tags.refreshAll   = refreshAll;
  
  return "utils-tagdata";
end); -- startup function  
