-- ------------------------------------------------------------------------------
-- RP Tags
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
--
-- The licensor cannot revoke these freedoms as long as you follow the
-- license terms.
--
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Utils-Tags: Utilities for creating tags in ElvUI or RPUF
  -- list of tags at the end

local RPTAGS     = RPTAGS;

table.insert(RPTAGS.startup, function(state) -- startup function
  if state.elvui
  then 
    local _G                = _G;
    local ElvUI             = _G["ElvUI"];
    local E, _, _, _, _, _  = unpack(ElvUI); 
    local UF                = E:GetModule('UnitFrames');
    
    if not RPTAGS.utils              then RPTAGS.utils              = {} end;
    if not RPTAGS.utils.elvui        then RPTAGS.utils.elvui        = {} end;
    if not RPTAGS.utils.elvui.tags   then RPTAGS.utils.elvui.tags   = {} end;
    if not RPTAGS.utils.elvui.frames then RPTAGS.utils.elvui.frames = {} end;
    
          -- registers one tag, an event to wait for, and a method to invoke when found --------------------------
    local function registerTag(tag)
      local  TagData = RPTAGS.cache.tagcache[tag];
      if     TagData.external -- this means it's a tag from outside of our program so we don't need to add it
      then   return false;
      else   local events = RPTAGS.const.MAIN_EVENT;
             if TagData.extraEvents then events = events .. " " .. TagData.extraEvents; end;
             _G["ElvUF"].Tags.Events[tag]  = events;
             _G["ElvUF"].Tags.Methods[tag] = TagData.method;
      end; -- if
    end; -- function
    
    local function registerAllTags() for tag, _ in pairs(RPTAGS.cache.tagcache) do registerTag(tag); end; end; -- function
    
    local function refreshFrame(frame)
      if     frame == "party" then UF:CreateAndUpdateHeaderGroup('party');
      elseif frame == "raid"  then UF:CreateAndUpdateHeaderGroup("raid")
      else                         UF:CreateAndUpdateUF(frame);
      end; -- if
    end;
    local function refreshAll() UF:Update_AllFrames(); end;

    RPTAGS.utils.elvui.tags.register     = registerTag; 
    RPTAGS.utils.elvui.tags.registerall  = registerAllTags;
    RPTAGS.utils.elvui.frames.refresh    = refreshFrame;
    RPTAGS.utils.elvui.frames.refreshAll = refreshAll;
  
    return "utils-elvui"
  else
    return "utils-elvui-disabled"
  end;

end); -- startup function
